@@ -1,4 +1,4 @@
-// Copyright (c) 2016-2019 The Bitcoin Core developers
+// Copyright (c) 2016-2020 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -10,7 +10,7 @@
 #include <vector>
 
 
-static void Base58Encode(benchmark::State& state)
+static void Base58Encode(benchmark::Bench& bench)
 {
     static const std::array<unsigned char, 32> buff = {
         {
@@ -19,13 +19,13 @@ static void Base58Encode(benchmark::State& state)
             200, 24
         }
     };
-    while (state.KeepRunning()) {
-        EncodeBase58(buff.data(), buff.data() + buff.size());
-    }
+    bench.batch(buff.size()).unit("byte").run([&] {
+        EncodeBase58(buff);
+    });
 }
 
 
-static void Base58CheckEncode(benchmark::State& state)
+static void Base58CheckEncode(benchmark::Bench& bench)
 {
     static const std::array<unsigned char, 32> buff = {
         {
@@ -34,24 +34,22 @@ static void Base58CheckEncode(benchmark::State& state)
             200, 24
         }
     };
-    std::vector<unsigned char> vch;
-    vch.assign(buff.begin(), buff.end());
-    while (state.KeepRunning()) {
-        EncodeBase58Check(vch);
-    }
+    bench.batch(buff.size()).unit("byte").run([&] {
+        EncodeBase58Check(buff);
+    });
 }
 
 
-static void Base58Decode(benchmark::State& state)
+static void Base58Decode(benchmark::Bench& bench)
 {
     const char* addr = "17VZNX1SN5NtKa8UQFxwQbFeFc3iqRYhem";
     std::vector<unsigned char> vch;
-    while (state.KeepRunning()) {
+    bench.batch(strlen(addr)).unit("byte").run([&] {
         (void) DecodeBase58(addr, vch, 64);
-    }
+    });
 }
 
 
-BENCHMARK(Base58Encode, 470 * 1000);
-BENCHMARK(Base58CheckEncode, 320 * 1000);
-BENCHMARK(Base58Decode, 800 * 1000);
+BENCHMARK(Base58Encode);
+BENCHMARK(Base58CheckEncode);
+BENCHMARK(Base58Decode);
