@@ -1,17 +1,17 @@
-// Copyright (c) 2015-2019 The Bitcoin Core developers
+// Copyright (c) 2015-2020 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
 #include <bench/bench.h>
-#include <util/system.h>
 #include <checkqueue.h>
+#include <key.h>
 #include <prevector.h>
-#include <vector>
-#include <boost/thread/thread.hpp>
+#include <pubkey.h>
 #include <random.h>
+#include <util/system.h>
 
+#include <vector>
 
-static const int MIN_CORES = 2;
 static const size_t BATCHES = 101;
 static const size_t BATCH_SIZE = 30;
 static const int PREVECTOR_SIZE = 28;
@@ -20,8 +20,14 @@ static const unsigned int QUEUE_BATCH_SIZE = 128;
 // This Benchmark tests the CheckQueue with a slightly realistic workload,
 // where checks all contain a prevector that is indirect 50% of the time
 // and there is a little bit of work done between calls to Add.
-static void CCheckQueueSpeedPrevectorJob(benchmark::State& state)
+static void CCheckQueueSpeedPrevectorJob(benchmark::Bench& bench)
 {
+    // We shouldn't ever be running with the checkqueue on a single core machine.
+    if (GetNumCores() <= 1) return;
+
+    const ECCVerifyHandle verify_handle;
+    ECC_Start();
+
     struct PrevectorJob {
         prevector<PREVECTOR_SIZE, uint8_t> p;
         PrevectorJob(){
@@ -36,26 +42,30 @@ static void CCheckQueueSpeedPrevectorJob(benchmark::State& state)
         void swap(PrevectorJob& x){p.swap(x.p);};
     };
     CCheckQueue<PrevectorJob> queue {QUEUE_BATCH_SIZE};
-    boost::thread_group tg;
-    for (auto x = 0; x < std::max(MIN_CORES, GetNumCores()); ++x) {
-       tg.create_thread([&]{queue.Thread();});
+    // The main thread should be counted to prevent thread oversubscription, and
+    // to decrease the variance of benchmark results.
+    queue.StartWorkerThreads(GetNumCores() - 1);
+
+    // create all the data once, then submit copies in the benchmark.
+    FastRandomContext insecure_rand(true);
+    std::vector<std::vector<PrevectorJob>> vBatches(BATCHES);
+    for (auto& vChecks : vBatches) {
+        vChecks.reserve(BATCH_SIZE);
+        for (size_t x = 0; x < BATCH_SIZE; ++x)
+            vChecks.emplace_back(insecure_rand);
     }
-    while (state.KeepRunning()) {
+
+    bench.minEpochIterations(10).batch(BATCH_SIZE * BATCHES).unit("job").run([&] {
         // Make insecure_rand here so that each iteration is identical.
-        FastRandomContext insecure_rand(true);
         CCheckQueueControl<PrevectorJob> control(&queue);
-        std::vector<std::vector<PrevectorJob>> vBatches(BATCHES);
-        for (auto& vChecks : vBatches) {
-            vChecks.reserve(BATCH_SIZE);
-            for (size_t x = 0; x < BATCH_SIZE; ++x)
-                vChecks.emplace_back(insecure_rand);
+        for (auto vChecks : vBatches) {
             control.Add(vChecks);
         }
         // control waits for completion by RAII, but
         // it is done explicitly here for clarity
         control.Wait();
-    }
-    tg.interrupt_all();
-    tg.join_all();
+    });
+    queue.StopWorkerThreads();
+    ECC_Stop();
 }
-BENCHMARK(CCheckQueueSpeedPrevectorJob, 1400);
+BENCHMARK(CCheckQueueSpeedPrevectorJob);
