@@ -1,5 +1,5 @@
 // Copyright (c) 2009-2010 Satoshi Nakamoto
-// Copyright (c) 2009-2019 The Bitcoin Core developers
+// Copyright (c) 2009-2020 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -22,29 +22,15 @@ public:
     //! during snapshot load to estimate progress of UTXO set reconstruction.
     uint64_t m_coins_count = 0;
 
-    //! Necessary to "fake" the base nChainTx so that we can estimate progress during
-    //! initial block download for the assumeutxo chainstate.
-    unsigned int m_nchaintx = 0;
-
     SnapshotMetadata() { }
     SnapshotMetadata(
         const uint256& base_blockhash,
         uint64_t coins_count,
         unsigned int nchaintx) :
             m_base_blockhash(base_blockhash),
-            m_coins_count(coins_count),
-            m_nchaintx(nchaintx) { }
-
-    ADD_SERIALIZE_METHODS;
-
-    template <typename Stream, typename Operation>
-    inline void SerializationOp(Stream& s, Operation ser_action)
-    {
-        READWRITE(m_base_blockhash);
-        READWRITE(m_coins_count);
-        READWRITE(m_nchaintx);
-    }
+            m_coins_count(coins_count) { }
 
+    SERIALIZE_METHODS(SnapshotMetadata, obj) { READWRITE(obj.m_base_blockhash, obj.m_coins_count); }
 };
 
 #endif // BITCOIN_NODE_UTXO_SNAPSHOT_H
