@@ -1,9 +1,9 @@
-<TS language="mk" version="2.1">
+<TS version="2.1" language="mk">
 <context>
     <name>AddressBookPage</name>
     <message>
         <source>Right-click to edit address or label</source>
-        <translation>Десен клик за уредување на адреса или етикета</translation>
+        <translation type="unfinished">Десен клик за уредување на адреса или етикета</translation>
     </message>
     <message>
         <source>Create a new address</source>
@@ -11,7 +11,7 @@
     </message>
     <message>
         <source>&amp;New</source>
-        <translation>&amp;Нова</translation>
+        <translation type="unfinished">&amp;Нова</translation>
     </message>
     <message>
         <source>Copy the currently selected address to the system clipboard</source>
@@ -19,11 +19,11 @@
     </message>
     <message>
         <source>&amp;Copy</source>
-        <translation>&amp;Копирај</translation>
+        <translation type="unfinished">&amp;Копирај</translation>
     </message>
     <message>
         <source>C&amp;lose</source>
-        <translation>З&amp;атвори</translation>
+        <translation type="unfinished">З&amp;атвори</translation>
     </message>
     <message>
         <source>Delete the currently selected address from the list</source>
@@ -42,9 +42,6 @@
         <translation>&amp;Избриши</translation>
     </message>
     </context>
-<context>
-    <name>AddressTableModel</name>
-    </context>
 <context>
     <name>AskPassphraseDialog</name>
     <message>
@@ -61,18 +58,98 @@
     </message>
     </context>
 <context>
-    <name>BanTableModel</name>
-    </context>
+    <name>QObject</name>
+    <message>
+        <source>Amount</source>
+        <translation type="unfinished">Сума</translation>
+    </message>
+    <message>
+        <source>%1 d</source>
+        <translation type="unfinished">%1 д</translation>
+    </message>
+    <message>
+        <source>%1 h</source>
+        <translation type="unfinished">%1 ч</translation>
+    </message>
+    <message>
+        <source>%1 m</source>
+        <translation type="unfinished">%1 м</translation>
+    </message>
+    <message>
+        <source>%1 s</source>
+        <translation type="unfinished">%1 с</translation>
+    </message>
+    <message>
+        <source>%1 ms</source>
+        <translation type="unfinished">%1 мс</translation>
+    </message>
+    <message numerus="yes">
+        <source>%n second(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n minute(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n hour(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n day(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n week(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message>
+        <source>%1 and %2</source>
+        <translation type="unfinished">%1 и %2</translation>
+    </message>
+    <message numerus="yes">
+        <source>%n year(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message>
+        <source>%1 B</source>
+        <translation type="unfinished">%1 Б</translation>
+    </message>
+    <message>
+        <source>%1 MB</source>
+        <translation type="unfinished">%1 МБ</translation>
+    </message>
+    <message>
+        <source>%1 GB</source>
+        <translation type="unfinished">%1 ГБ</translation>
+    </message>
+</context>
 <context>
     <name>BitcoinGUI</name>
-    <message>
-        <source>Sign &amp;message...</source>
-        <translation>Потпиши &amp;порака...</translation>
-    </message>
-    <message>
-        <source>Synchronizing with network...</source>
-        <translation>Синхронизација со мрежата...</translation>
-    </message>
     <message>
         <source>&amp;Overview</source>
         <translation>&amp;Преглед</translation>
@@ -101,38 +178,10 @@
         <source>Show information about Qt</source>
         <translation>Прикажи информации за Qt</translation>
     </message>
-    <message>
-        <source>&amp;Options...</source>
-        <translation>&amp;Опции...</translation>
-    </message>
-    <message>
-        <source>&amp;Encrypt Wallet...</source>
-        <translation>&amp;Криптирање на Паричник...</translation>
-    </message>
-    <message>
-        <source>&amp;Backup Wallet...</source>
-        <translation>&amp;Бекап на Паричник...</translation>
-    </message>
-    <message>
-        <source>&amp;Change Passphrase...</source>
-        <translation>&amp;Измени Тајна Фраза...</translation>
-    </message>
-    <message>
-        <source>Open &amp;URI...</source>
-        <translation>Отвори &amp;URI...</translation>
-    </message>
-    <message>
-        <source>Reindexing blocks on disk...</source>
-        <translation>Повторно индексирање на блокови од дискот...</translation>
-    </message>
     <message>
         <source>Send coins to a Bitcoin address</source>
         <translation>Испрати биткоини на Биткоин адреса</translation>
     </message>
-    <message>
-        <source>&amp;Verify message...</source>
-        <translation>&amp;Потврди порака...</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Испрати</translation>
@@ -159,7 +208,11 @@
     </message>
     <message numerus="yes">
         <source>Processed %n block(s) of transaction history.</source>
-        <translation><numerusform>Обработен %n блок од историјата на трансакции.</numerusform><numerusform>Обработени %n блокови од историјата на трансакции.</numerusform></translation>
+        <translation>
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
     <message>
         <source>%1 behind</source>
@@ -179,36 +232,45 @@
     </message>
     <message>
         <source>&amp;Window</source>
-        <translation>&amp;Прозорец</translation>
+        <translation type="unfinished">&amp;Прозорец</translation>
+    </message>
+    <message numerus="yes">
+        <source>%n active connection(s) to Bitcoin network.</source>
+        <extracomment>A substring of the tooltip.</extracomment>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
     <message>
         <source>Date: %1
 </source>
-        <translation>Дата: %1
+        <translation type="unfinished">Дата: %1
 </translation>
     </message>
     <message>
         <source>Amount: %1
 </source>
-        <translation>Сума: %1
+        <translation type="unfinished">Сума: %1
 </translation>
     </message>
     <message>
         <source>Type: %1
 </source>
-        <translation>Тип: %1
+        <translation type="unfinished">Тип: %1
 </translation>
     </message>
     <message>
         <source>Label: %1
 </source>
-        <translation>Етикета: %1
+        <translation type="unfinished">Етикета: %1
 </translation>
     </message>
     <message>
         <source>Address: %1
 </source>
-        <translation>Адреса: %1
+        <translation type="unfinished">Адреса: %1
 </translation>
     </message>
     </context>
@@ -216,45 +278,42 @@
     <name>CoinControlDialog</name>
     <message>
         <source>Bytes:</source>
-        <translation>Бајти:</translation>
+        <translation type="unfinished">Бајти:</translation>
     </message>
     <message>
         <source>Amount:</source>
-        <translation>Сума:</translation>
+        <translation type="unfinished">Сума:</translation>
     </message>
     <message>
         <source>Fee:</source>
-        <translation>Провизија:</translation>
+        <translation type="unfinished">Провизија:</translation>
     </message>
     <message>
         <source>Dust:</source>
-        <translation>Прашина:</translation>
+        <translation type="unfinished">Прашина:</translation>
     </message>
     <message>
         <source>After Fee:</source>
-        <translation>После Провизија:</translation>
+        <translation type="unfinished">После Провизија:</translation>
     </message>
     <message>
         <source>Change:</source>
-        <translation>Кусур:</translation>
+        <translation type="unfinished">Кусур:</translation>
     </message>
     <message>
         <source>Amount</source>
-        <translation>Сума</translation>
+        <translation type="unfinished">Сума</translation>
     </message>
     <message>
         <source>Date</source>
-        <translation>Дата</translation>
+        <translation type="unfinished">Дата</translation>
     </message>
     </context>
-<context>
-    <name>CreateWalletActivity</name>
-    </context>
 <context>
     <name>CreateWalletDialog</name>
     <message>
         <source>Wallet</source>
-        <translation>Паричник</translation>
+        <translation type="unfinished">Паричник</translation>
     </message>
     </context>
 <context>
@@ -279,18 +338,20 @@
         <translation>име</translation>
     </message>
     </context>
-<context>
-    <name>HelpMessageDialog</name>
-    <message>
-        <source>version</source>
-        <translation>верзија</translation>
-    </message>
-    </context>
 <context>
     <name>Intro</name>
     <message>
         <source>Bitcoin</source>
-        <translation>Биткоин</translation>
+        <translation type="unfinished">Биткоин</translation>
+    </message>
+    <message numerus="yes">
+        <source>(sufficient to restore backups %n day(s) old)</source>
+        <extracomment>Explanatory text on the capability of the current prune target.</extracomment>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
     <message>
         <source>Error</source>
@@ -298,17 +359,11 @@
     </message>
     </context>
 <context>
-    <name>ModalOverlay</name>
-    </context>
-<context>
-    <name>OpenURIDialog</name>
+    <name>HelpMessageDialog</name>
     <message>
-        <source>URI:</source>
-        <translation>URI:</translation>
+        <source>version</source>
+        <translation type="unfinished">верзија</translation>
     </message>
-</context>
-<context>
-    <name>OpenWalletActivity</name>
     </context>
 <context>
     <name>OptionsDialog</name>
@@ -322,7 +377,7 @@
     </message>
     <message>
         <source>W&amp;allet</source>
-        <translation>П&amp;аричник</translation>
+        <translation type="unfinished">П&amp;аричник</translation>
     </message>
     <message>
         <source>&amp;Window</source>
@@ -338,11 +393,11 @@
     </message>
     <message>
         <source>none</source>
-        <translation>нема</translation>
+        <translation type="unfinished">нема</translation>
     </message>
     <message>
         <source>Error</source>
-        <translation>Грешка</translation>
+        <translation type="unfinished">Грешка</translation>
     </message>
     </context>
 <context>
@@ -352,73 +407,19 @@
         <translation>Вкупно:</translation>
     </message>
     </context>
-<context>
-    <name>PSBTOperationsDialog</name>
-    </context>
-<context>
-    <name>PaymentServer</name>
-    </context>
 <context>
     <name>PeerTableModel</name>
     <message>
         <source>Sent</source>
-        <translation>Испратени</translation>
-    </message>
-    </context>
-<context>
-    <name>QObject</name>
-    <message>
-        <source>Amount</source>
-        <translation>Сума</translation>
+        <extracomment>Title of Peers Table column which indicates the total amount of network information we have sent to the peer.</extracomment>
+        <translation type="unfinished">Испратени</translation>
     </message>
     <message>
-        <source>%1 d</source>
-        <translation>%1 д</translation>
+        <source>Network</source>
+        <extracomment>Title of Peers Table column which states the network the peer connected through.</extracomment>
+        <translation type="unfinished">Мрежа</translation>
     </message>
-    <message>
-        <source>%1 h</source>
-        <translation>%1 ч</translation>
-    </message>
-    <message>
-        <source>%1 m</source>
-        <translation>%1 м</translation>
-    </message>
-    <message>
-        <source>%1 s</source>
-        <translation>%1 с</translation>
-    </message>
-    <message>
-        <source>%1 ms</source>
-        <translation>%1 мс</translation>
-    </message>
-    <message>
-        <source>%1 and %2</source>
-        <translation>%1 и %2</translation>
-    </message>
-    <message>
-        <source>%1 B</source>
-        <translation>%1 Б</translation>
-    </message>
-    <message>
-        <source>%1 KB</source>
-        <translation>%1 КБ</translation>
-    </message>
-    <message>
-        <source>%1 MB</source>
-        <translation>%1 МБ</translation>
-    </message>
-    <message>
-        <source>%1 GB</source>
-        <translation>%1 ГБ</translation>
-    </message>
-    </context>
-<context>
-    <name>QRImageWidget</name>
-    <message>
-        <source>&amp;Save Image...</source>
-        <translation>&amp;Сними Слика...</translation>
-    </message>
-    </context>
+</context>
 <context>
     <name>RPCConsole</name>
     <message>
@@ -427,23 +428,19 @@
     </message>
     <message>
         <source>Name</source>
-        <translation>Име</translation>
+        <translation type="unfinished">Име</translation>
     </message>
     <message>
         <source>Number of connections</source>
         <translation>Број на конекции</translation>
     </message>
-    <message>
-        <source>Block chain</source>
-        <translation>Block chain</translation>
-    </message>
     <message>
         <source>Sent</source>
-        <translation>Испратени</translation>
+        <translation type="unfinished">Испратени</translation>
     </message>
     <message>
         <source>Version</source>
-        <translation>Верзија</translation>
+        <translation type="unfinished">Верзија</translation>
     </message>
     <message>
         <source>&amp;Console</source>
@@ -454,76 +451,84 @@
     <name>ReceiveCoinsDialog</name>
     <message>
         <source>&amp;Amount:</source>
-        <translation>&amp;Сума:</translation>
+        <translation type="unfinished">&amp;Сума:</translation>
     </message>
     <message>
         <source>&amp;Label:</source>
-        <translation>&amp;Етикета:</translation>
+        <translation type="unfinished">&amp;Етикета:</translation>
     </message>
     <message>
         <source>&amp;Message:</source>
-        <translation>&amp;Порака:</translation>
+        <translation type="unfinished">&amp;Порака:</translation>
     </message>
     <message>
         <source>Show</source>
-        <translation>Прикажи</translation>
+        <translation type="unfinished">Прикажи</translation>
+    </message>
+    <message>
+        <source>Copy &amp;URI</source>
+        <translation type="unfinished">Копирај &amp;URI</translation>
     </message>
     </context>
 <context>
     <name>ReceiveRequestDialog</name>
     <message>
         <source>Amount:</source>
-        <translation>Сума:</translation>
+        <translation type="unfinished">Сума:</translation>
     </message>
     <message>
         <source>Message:</source>
-        <translation>Порака:</translation>
+        <translation type="unfinished">Порака:</translation>
     </message>
     <message>
         <source>Copy &amp;URI</source>
-        <translation>Копирај &amp;URI</translation>
+        <translation type="unfinished">Копирај &amp;URI</translation>
     </message>
     <message>
         <source>Copy &amp;Address</source>
-        <translation>Копирај &amp;Адреса</translation>
-    </message>
-    <message>
-        <source>&amp;Save Image...</source>
-        <translation>&amp;Сними Слика...</translation>
+        <translation type="unfinished">Копирај &amp;Адреса</translation>
     </message>
     </context>
 <context>
     <name>RecentRequestsTableModel</name>
     <message>
         <source>Date</source>
-        <translation>Дата</translation>
+        <translation type="unfinished">Дата</translation>
     </message>
     </context>
 <context>
     <name>SendCoinsDialog</name>
     <message>
         <source>Bytes:</source>
-        <translation>Бајти:</translation>
+        <translation type="unfinished">Бајти:</translation>
     </message>
     <message>
         <source>Amount:</source>
-        <translation>Сума:</translation>
+        <translation type="unfinished">Сума:</translation>
     </message>
     <message>
         <source>Fee:</source>
-        <translation>Провизија:</translation>
+        <translation type="unfinished">Провизија:</translation>
     </message>
     <message>
         <source>After Fee:</source>
-        <translation>После Провизија:</translation>
+        <translation type="unfinished">После Провизија:</translation>
     </message>
     <message>
         <source>Change:</source>
-        <translation>Кусур:</translation>
+        <translation type="unfinished">Кусур:</translation>
     </message>
     <message>
         <source>Dust:</source>
-        <translation>Прашина:</translation>
+        <translation type="unfinished">Прашина:</translation>
+    </message>
+    <message numerus="yes">
+        <source>Estimated to begin confirmation within %n block(s).</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
     </context>
 <context>
@@ -538,74 +543,71 @@
     </message>
     <message>
         <source>Message:</source>
-        <translation>Порака:</translation>
+        <translation type="unfinished">Порака:</translation>
     </message>
     </context>
-<context>
-    <name>ShutdownWindow</name>
-    </context>
-<context>
-    <name>SignVerifyMessageDialog</name>
-    </context>
-<context>
-    <name>TrafficGraphWidget</name>
-    </context>
 <context>
     <name>TransactionDesc</name>
+    <message numerus="yes">
+        <source>Open for %n more block(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
     <message>
         <source>Date</source>
-        <translation>Дата</translation>
+        <translation type="unfinished">Дата</translation>
+    </message>
+    <message numerus="yes">
+        <source>matures in %n more block(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
     <message>
         <source>Amount</source>
-        <translation>Сума</translation>
+        <translation type="unfinished">Сума</translation>
     </message>
     </context>
-<context>
-    <name>TransactionDescDialog</name>
-    </context>
 <context>
     <name>TransactionTableModel</name>
     <message>
         <source>Date</source>
-        <translation>Дата</translation>
+        <translation type="unfinished">Дата</translation>
+    </message>
+    <message numerus="yes">
+        <source>Open for %n more block(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
     </context>
 <context>
     <name>TransactionView</name>
     <message>
         <source>Date</source>
-        <translation>Дата</translation>
+        <translation type="unfinished">Дата</translation>
     </message>
     </context>
-<context>
-    <name>UnitDisplayStatusBarControl</name>
-    </context>
-<context>
-    <name>WalletController</name>
-    </context>
-<context>
-    <name>WalletFrame</name>
-    </context>
-<context>
-    <name>WalletModel</name>
-    </context>
 <context>
     <name>WalletView</name>
     <message>
         <source>&amp;Export</source>
-        <translation>&amp;Експорт</translation>
+        <translation type="unfinished">&amp;Експорт</translation>
     </message>
     <message>
         <source>Export the data in the current tab to a file</source>
-        <translation>Експортирај ги податоците од активното јазиче во датотека</translation>
+        <translation type="unfinished">Експортирај ги податоците од активното јазиче во датотека</translation>
     </message>
     <message>
         <source>Error</source>
-        <translation>Грешка</translation>
+        <translation type="unfinished">Грешка</translation>
     </message>
     </context>
-<context>
-    <name>bitcoin-core</name>
-    </context>
 </TS>
\ No newline at end of file
