@@ -1,9 +1,9 @@
-<TS language="ms" version="2.1">
+<TS version="2.1" language="ms">
 <context>
     <name>AddressBookPage</name>
     <message>
         <source>Right-click to edit address or label</source>
-        <translation>Klik-kanan untuk edit alamat ataupun label</translation>
+        <translation type="unfinished">Klik-kanan untuk edit alamat ataupun label</translation>
     </message>
     <message>
         <source>Create a new address</source>
@@ -11,7 +11,7 @@
     </message>
     <message>
         <source>&amp;New</source>
-        <translation>&amp;Baru</translation>
+        <translation type="unfinished">&amp;Baru</translation>
     </message>
     <message>
         <source>Copy the currently selected address to the system clipboard</source>
@@ -19,11 +19,11 @@
     </message>
     <message>
         <source>&amp;Copy</source>
-        <translation>&amp;Salin</translation>
+        <translation type="unfinished">&amp;Salin</translation>
     </message>
     <message>
         <source>C&amp;lose</source>
-        <translation>&amp;Tutup</translation>
+        <translation type="unfinished">&amp;Tutup</translation>
     </message>
     <message>
         <source>Delete the currently selected address from the list</source>
@@ -31,7 +31,7 @@
     </message>
     <message>
         <source>Enter address or label to search</source>
-        <translation>Masukkan alamat atau label untuk carian
+        <translation type="unfinished">Masukkan alamat atau label untuk carian
 </translation>
     </message>
     <message>
@@ -49,70 +49,59 @@ Alihkan fail data ke dalam tab semasa</translation>
     </message>
     <message>
         <source>Choose the address to send coins to</source>
-        <translation>Pilih alamat untuk hantar koin kepada</translation>
+        <translation type="unfinished">Pilih alamat untuk hantar koin kepada</translation>
     </message>
     <message>
         <source>Choose the address to receive coins with</source>
-        <translation>Pilih alamat untuk menerima koin dengan</translation>
+        <translation type="unfinished">Pilih alamat untuk menerima koin dengan</translation>
     </message>
     <message>
         <source>C&amp;hoose</source>
-        <translation>&amp;Pilih</translation>
+        <translation type="unfinished">&amp;Pilih</translation>
     </message>
     <message>
         <source>Sending addresses</source>
-        <translation>alamat-alamat penghantaran</translation>
+        <translation type="unfinished">alamat-alamat penghantaran</translation>
     </message>
     <message>
         <source>Receiving addresses</source>
-        <translation>alamat-alamat penerimaan</translation>
+        <translation type="unfinished">alamat-alamat penerimaan</translation>
     </message>
     <message>
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
-        <translation>Ini adalah alamat Bitcoin anda untuk pembayaran. Periksa jumlah dan alamat penerima sebelum membuat penghantaran koin sentiasa.</translation>
+        <translation type="unfinished">Ini adalah alamat Bitcoin anda untuk pembayaran. Periksa jumlah dan alamat penerima sebelum membuat penghantaran koin sentiasa.</translation>
     </message>
     <message>
         <source>&amp;Copy Address</source>
-        <translation>&amp;Salin Aamat</translation>
+        <translation type="unfinished">&amp;Salin Aamat</translation>
     </message>
     <message>
         <source>Copy &amp;Label</source>
-        <translation>Salin &amp; Label</translation>
-    </message>
-    <message>
-        <source>&amp;Edit</source>
-        <translation>&amp;Edit</translation>
+        <translation type="unfinished">Salin &amp; Label</translation>
     </message>
     <message>
         <source>Export Address List</source>
-        <translation>Eskport Senarai Alamat</translation>
-    </message>
-    <message>
-        <source>Comma separated file (*.csv)</source>
-        <translation>Fail dibahagi oleh koma(*.csv)</translation>
-    </message>
-    <message>
-        <source>Exporting Failed</source>
-        <translation>Mengeksport Gagal</translation>
+        <translation type="unfinished">Eskport Senarai Alamat</translation>
     </message>
     <message>
         <source>There was an error trying to save the address list to %1. Please try again.</source>
-        <translation>Terdapat ralat semasa cubaan menyimpan senarai alamat kepada %1. Sila cuba lagi.</translation>
+        <extracomment>An error message. %1 is a stand-in argument for the name of the file we attempted to save to.</extracomment>
+        <translation type="unfinished">Terdapat ralat semasa cubaan menyimpan senarai alamat kepada %1. Sila cuba lagi.</translation>
+    </message>
+    <message>
+        <source>Exporting Failed</source>
+        <translation type="unfinished">Mengeksport Gagal</translation>
     </message>
 </context>
 <context>
     <name>AddressTableModel</name>
-    <message>
-        <source>Label</source>
-        <translation>Label</translation>
-    </message>
     <message>
         <source>Address</source>
-        <translation>Alamat</translation>
+        <translation type="unfinished">Alamat</translation>
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(tiada label)</translation>
+        <translation type="unfinished">(tiada label)</translation>
     </message>
 </context>
 <context>
@@ -135,102 +124,117 @@ Alihkan fail data ke dalam tab semasa</translation>
     </message>
     <message>
         <source>Encrypt wallet</source>
-        <translation>Dompet encrypt</translation>
+        <translation type="unfinished">Dompet encrypt</translation>
     </message>
     <message>
         <source>This operation needs your wallet passphrase to unlock the wallet.</source>
-        <translation>Operasi ini perlukan frasa laluan dompet anda untuk membuka kunci dompet.</translation>
+        <translation type="unfinished">Operasi ini perlukan frasa laluan dompet anda untuk membuka kunci dompet.</translation>
     </message>
     <message>
         <source>Unlock wallet</source>
-        <translation>Membuka kunci dompet</translation>
-    </message>
-    <message>
-        <source>This operation needs your wallet passphrase to decrypt the wallet.</source>
-        <translation>Operasi ini memerlukan frasa laluan dompet anda untuk menyahsulit dompet.</translation>
-    </message>
-    <message>
-        <source>Decrypt wallet</source>
-        <translation>Menyahsulit dompet</translation>
+        <translation type="unfinished">Membuka kunci dompet</translation>
     </message>
     <message>
         <source>Change passphrase</source>
-        <translation>Menukar frasa laluan</translation>
+        <translation type="unfinished">Menukar frasa laluan</translation>
     </message>
     <message>
         <source>Confirm wallet encryption</source>
-        <translation>Mengesahkan enkripsi dompet</translation>
+        <translation type="unfinished">Mengesahkan enkripsi dompet</translation>
     </message>
     <message>
         <source>Warning: If you encrypt your wallet and lose your passphrase, you will &lt;b&gt;LOSE ALL OF YOUR BITCOINS&lt;/b&gt;!</source>
-        <translation>Amaran: Jika anda enkripkan dompet anda dan hilangkan frasa laluan, anda akan &lt;b&gt;ANDA AKAN HILANGKAN SEMUA BITCOIN ANDA&lt;/b&gt;!</translation>
+        <translation type="unfinished">Amaran: Jika anda enkripkan dompet anda dan hilangkan frasa laluan, anda akan &lt;b&gt;ANDA AKAN HILANGKAN SEMUA BITCOIN ANDA&lt;/b&gt;!</translation>
     </message>
     <message>
         <source>Are you sure you wish to encrypt your wallet?</source>
-        <translation>Anda pasti untuk membuat enkripsi dompet anda?</translation>
+        <translation type="unfinished">Anda pasti untuk membuat enkripsi dompet anda?</translation>
     </message>
     <message>
         <source>Wallet encrypted</source>
-        <translation>Dompet dienkripsi</translation>
+        <translation type="unfinished">Dompet dienkripsi</translation>
     </message>
     <message>
         <source>IMPORTANT: Any previous backups you have made of your wallet file should be replaced with the newly generated, encrypted wallet file. For security reasons, previous backups of the unencrypted wallet file will become useless as soon as you start using the new, encrypted wallet.</source>
-        <translation>PENTING: Apa-apa sandaran yang anda buat sebelum ini untuk fail dompet anda hendaklah digantikan dengan fail dompet enkripsi yang dijana baru. Untuk sebab-sebab keselamatan , sandaran fail dompet yang belum dibuat enkripsi sebelum ini akan menjadi tidak berguna secepat anda mula guna dompet enkripsi baru.</translation>
+        <translation type="unfinished">PENTING: Apa-apa sandaran yang anda buat sebelum ini untuk fail dompet anda hendaklah digantikan dengan fail dompet enkripsi yang dijana baru. Untuk sebab-sebab keselamatan , sandaran fail dompet yang belum dibuat enkripsi sebelum ini akan menjadi tidak berguna secepat anda mula guna dompet enkripsi baru.</translation>
     </message>
     <message>
         <source>Wallet encryption failed</source>
-        <translation>Enkripsi dompet gagal</translation>
+        <translation type="unfinished">Enkripsi dompet gagal</translation>
     </message>
     <message>
         <source>Wallet encryption failed due to an internal error. Your wallet was not encrypted.</source>
-        <translation>Enkripsi dompet gagal kerana ralat dalaman. Dompet anda tidak dienkripkan.</translation>
+        <translation type="unfinished">Enkripsi dompet gagal kerana ralat dalaman. Dompet anda tidak dienkripkan.</translation>
     </message>
     <message>
         <source>The supplied passphrases do not match.</source>
-        <translation>Frasa laluan yang dibekalkan tidak sepadan.</translation>
+        <translation type="unfinished">Frasa laluan yang dibekalkan tidak sepadan.</translation>
     </message>
     <message>
         <source>Wallet unlock failed</source>
-        <translation>Pembukaan kunci dompet gagal</translation>
+        <translation type="unfinished">Pembukaan kunci dompet gagal</translation>
     </message>
     <message>
         <source>The passphrase entered for the wallet decryption was incorrect.</source>
-        <translation>Frasa laluan dimasukki untuk dekripsi dompet adalah tidak betul.</translation>
-    </message>
-    <message>
-        <source>Wallet decryption failed</source>
-        <translation>Dekripsi dompet gagal</translation>
+        <translation type="unfinished">Frasa laluan dimasukki untuk dekripsi dompet adalah tidak betul.</translation>
     </message>
     <message>
         <source>Wallet passphrase was successfully changed.</source>
-        <translation>Frasa laluan dompet berjaya ditukar.</translation>
+        <translation type="unfinished">Frasa laluan dompet berjaya ditukar.</translation>
     </message>
     <message>
         <source>Warning: The Caps Lock key is on!</source>
-        <translation>Amaran: Kunci Caps Lock buka!</translation>
+        <translation type="unfinished">Amaran: Kunci Caps Lock buka!</translation>
     </message>
 </context>
 <context>
     <name>BanTableModel</name>
-    <message>
-        <source>IP/Netmask</source>
-        <translation>IP/Netmask</translation>
-    </message>
     <message>
         <source>Banned Until</source>
-        <translation>Diharamkan sehingga</translation>
+        <translation type="unfinished">Diharamkan sehingga</translation>
     </message>
 </context>
+<context>
+    <name>QObject</name>
+    <message numerus="yes">
+        <source>%n second(s)</source>
+        <translation>
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n minute(s)</source>
+        <translation>
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n hour(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n day(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n week(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n year(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+        </translation>
+    </message>
+    </context>
 <context>
     <name>BitcoinGUI</name>
-    <message>
-        <source>Sign &amp;message...</source>
-        <translation>Tandatangan &amp; mesej...</translation>
-    </message>
-    <message>
-        <source>Synchronizing with network...</source>
-        <translation>Penyegerakan dengan rangkaian...</translation>
-    </message>
     <message>
         <source>&amp;Overview</source>
         <translation>&amp;Gambaran Keseluruhan</translation>
@@ -257,11 +261,11 @@ Alihkan fail data ke dalam tab semasa</translation>
     </message>
     <message>
         <source>&amp;About %1</source>
-        <translation>&amp;Mengenai%1</translation>
+        <translation type="unfinished">&amp;Mengenai%1</translation>
     </message>
     <message>
         <source>Show information about %1</source>
-        <translation>Menunjuk informasi mengenai%1</translation>
+        <translation type="unfinished">Menunjuk informasi mengenai%1</translation>
     </message>
     <message>
         <source>About &amp;Qt</source>
@@ -271,53 +275,18 @@ Alihkan fail data ke dalam tab semasa</translation>
         <source>Show information about Qt</source>
         <translation>Menunjuk informasi megenai Qt</translation>
     </message>
-    <message>
-        <source>&amp;Options...</source>
-        <translation>Pilihan</translation>
-    </message>
     <message>
         <source>Modify configuration options for %1</source>
-        <translation>Mengubah suai pilihan konfigurasi untuk %1</translation>
-    </message>
-    <message>
-        <source>&amp;Encrypt Wallet...</source>
-        <translation>&amp;Enkripsi Dompet</translation>
-    </message>
-    <message>
-        <source>&amp;Backup Wallet...</source>
-        <translation>&amp;Dompet Sandaran...</translation>
-    </message>
-    <message>
-        <source>&amp;Change Passphrase...</source>
-        <translation>&amp;Menukar frasa-laluan</translation>
-    </message>
-    <message>
-        <source>Open &amp;URI...</source>
-        <translation>Buka &amp;URI...</translation>
+        <translation type="unfinished">Mengubah suai pilihan konfigurasi untuk %1</translation>
     </message>
     <message>
         <source>Wallet:</source>
-        <translation>dompet</translation>
-    </message>
-    <message>
-        <source>Click to disable network activity.</source>
-        <translation>Tekan untuk lumpuhkan rangkaian</translation>
+        <translation type="unfinished">dompet</translation>
     </message>
     <message>
         <source>Network activity disabled.</source>
-        <translation>Aktiviti rangkaian dilumpuhkan</translation>
-    </message>
-    <message>
-        <source>Click to enable network activity again.</source>
-        <translation>Tekan untuk mengaktifkan rangkain semula</translation>
-    </message>
-    <message>
-        <source>Syncing Headers (%1%)...</source>
-        <translation>Penyelarasn tajuk (%1%)...</translation>
-    </message>
-    <message>
-        <source>Reindexing blocks on disk...</source>
-        <translation>Reindexi blok pada cakera...</translation>
+        <extracomment>A substring of the tooltip.</extracomment>
+        <translation type="unfinished">Aktiviti rangkaian dilumpuhkan</translation>
     </message>
     <message>
         <source>Send coins to a Bitcoin address</source>
@@ -331,10 +300,6 @@ Alihkan fail data ke dalam tab semasa</translation>
         <source>Change the passphrase used for wallet encryption</source>
         <translation>Tukar kata laluan untuk dompet disulitkan</translation>
     </message>
-    <message>
-        <source>&amp;Verify message...</source>
-        <translation>sahkan mesej</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>hantar</translation>
@@ -382,14 +347,20 @@ Alihkan fail data ke dalam tab semasa</translation>
     </message>
     <message>
         <source>Request payments (generates QR codes and bitcoin: URIs)</source>
-        <translation>Request payments (generates QR codes and bitcoin: URIs)
+        <translation type="unfinished">Request payments (generates QR codes and bitcoin: URIs)
 </translation>
     </message>
     <message>
         <source>Show the list of used sending addresses and labels</source>
-        <translation>Tunjukkan senarai alamat dan label yang digunakan
+        <translation type="unfinished">Tunjukkan senarai alamat dan label yang digunakan
 </translation>
     </message>
+    <message numerus="yes">
+        <source>Processed %n block(s) of transaction history.</source>
+        <translation>
+            <numerusform />
+        </translation>
+    </message>
     <message>
         <source>Error</source>
         <translation>Ralat</translation>
@@ -408,38 +379,53 @@ Alihkan fail data ke dalam tab semasa</translation>
     </message>
     <message>
         <source>Open Wallet</source>
-        <translation>Buka Wallet</translation>
-    </message>
-    <message>
-        <source>Close Wallet...</source>
-        <translation>Tutup Wallet...</translation>
+        <translation type="unfinished">Buka Wallet</translation>
     </message>
     <message>
         <source>Close wallet</source>
-        <translation>Tutup Wallet</translation>
+        <translation type="unfinished">Tutup Wallet</translation>
     </message>
     <message>
         <source>default wallet</source>
-        <translation>dompet lalai
+        <translation type="unfinished">dompet lalai
 </translation>
     </message>
+    <message numerus="yes">
+        <source>%n active connection(s) to Bitcoin network.</source>
+        <extracomment>A substring of the tooltip.</extracomment>
+        <translation type="unfinished">
+            <numerusform />
+        </translation>
+    </message>
     </context>
 <context>
     <name>CoinControlDialog</name>
-    <message>
-        <source>Amount</source>
-        <translation>Amount</translation>
-    </message>
     <message>
         <source>(no label)</source>
-        <translation>(tiada label)</translation>
+        <translation type="unfinished">(tiada label)</translation>
     </message>
     </context>
 <context>
-    <name>CreateWalletActivity</name>
+    <name>OpenWalletActivity</name>
+    <message>
+        <source>default wallet</source>
+        <translation type="unfinished">dompet lalai
+</translation>
+    </message>
+    </context>
+<context>
+    <name>WalletController</name>
+    <message>
+        <source>Close wallet</source>
+        <translation type="unfinished">Tutup Wallet</translation>
+    </message>
     </context>
 <context>
     <name>CreateWalletDialog</name>
+    <message>
+        <source>Wallet</source>
+        <translation type="unfinished">dompet</translation>
+    </message>
     </context>
 <context>
     <name>EditAddressDialog</name>
@@ -452,105 +438,51 @@ Alihkan fail data ke dalam tab semasa</translation>
         <translation>Alamat</translation>
     </message>
     </context>
-<context>
-    <name>FreespaceChecker</name>
-    </context>
-<context>
-    <name>HelpMessageDialog</name>
-    </context>
 <context>
     <name>Intro</name>
-    <message>
-        <source>Bitcoin</source>
-        <translation>Bitcoin</translation>
+    <message numerus="yes">
+        <source>(sufficient to restore backups %n day(s) old)</source>
+        <extracomment>Explanatory text on the capability of the current prune target.</extracomment>
+        <translation type="unfinished">
+            <numerusform />
+        </translation>
     </message>
     <message>
         <source>Error</source>
         <translation>Ralat</translation>
     </message>
     </context>
-<context>
-    <name>ModalOverlay</name>
-    </context>
-<context>
-    <name>OpenURIDialog</name>
-    </context>
-<context>
-    <name>OpenWalletActivity</name>
-    <message>
-        <source>default wallet</source>
-        <translation>dompet lalai
-</translation>
-    </message>
-    <message>
-        <source>Opening Wallet &lt;b&gt;%1&lt;/b&gt;...</source>
-        <translation>Buka sedang Wallet &lt;b&gt;%1&lt;/b&gt;...</translation>
-    </message>
-</context>
 <context>
     <name>OptionsDialog</name>
     <message>
         <source>Error</source>
-        <translation>Ralat</translation>
+        <translation type="unfinished">Ralat</translation>
     </message>
     </context>
-<context>
-    <name>OverviewPage</name>
-    </context>
-<context>
-    <name>PaymentServer</name>
-    </context>
 <context>
     <name>PeerTableModel</name>
-    </context>
-<context>
-    <name>QObject</name>
     <message>
-        <source>Amount</source>
-        <translation>Amount</translation>
+        <source>Address</source>
+        <extracomment>Title of Peers Table column which contains the IP/Onion/I2P address of the connected peer.</extracomment>
+        <translation type="unfinished">Alamat</translation>
     </message>
     </context>
-<context>
-    <name>QRImageWidget</name>
-    </context>
-<context>
-    <name>RPCConsole</name>
-    </context>
-<context>
-    <name>ReceiveCoinsDialog</name>
-    </context>
 <context>
     <name>ReceiveRequestDialog</name>
+    <message>
+        <source>Wallet:</source>
+        <translation type="unfinished">dompet</translation>
+    </message>
     <message>
         <source>Copy &amp;Address</source>
-        <translation>&amp;Salin Alamat</translation>
+        <translation type="unfinished">&amp;Salin Alamat</translation>
     </message>
-    <message>
-        <source>Address</source>
-        <translation>Alamat</translation>
-    </message>
-    <message>
-        <source>Amount</source>
-        <translation>Amount</translation>
-    </message>
-    <message>
-        <source>Label</source>
-        <translation>Label</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>dompet</translation>
-    </message>
-</context>
+    </context>
 <context>
     <name>RecentRequestsTableModel</name>
-    <message>
-        <source>Label</source>
-        <translation>Label</translation>
-    </message>
     <message>
         <source>(no label)</source>
-        <translation>(tiada label)</translation>
+        <translation type="unfinished">(tiada label)</translation>
     </message>
     </context>
 <context>
@@ -559,81 +491,61 @@ Alihkan fail data ke dalam tab semasa</translation>
         <source>Balance:</source>
         <translation>Baki</translation>
     </message>
+    <message numerus="yes">
+        <source>Estimated to begin confirmation within %n block(s).</source>
+        <translation>
+            <numerusform />
+        </translation>
+    </message>
     <message>
         <source>(no label)</source>
-        <translation>(tiada label)</translation>
+        <translation type="unfinished">(tiada label)</translation>
     </message>
 </context>
-<context>
-    <name>SendCoinsEntry</name>
-    </context>
-<context>
-    <name>ShutdownWindow</name>
-    </context>
-<context>
-    <name>SignVerifyMessageDialog</name>
-    </context>
-<context>
-    <name>TrafficGraphWidget</name>
-    </context>
 <context>
     <name>TransactionDesc</name>
-    <message>
-        <source>Amount</source>
-        <translation>Amount</translation>
+    <message numerus="yes">
+        <source>Open for %n more block(s)</source>
+        <translation>
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>matures in %n more block(s)</source>
+        <translation>
+            <numerusform />
+        </translation>
     </message>
-    </context>
-<context>
-    <name>TransactionDescDialog</name>
     </context>
 <context>
     <name>TransactionTableModel</name>
-    <message>
-        <source>Label</source>
-        <translation>Label</translation>
+    <message numerus="yes">
+        <source>Open for %n more block(s)</source>
+        <translation>
+            <numerusform />
+        </translation>
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(tiada label)</translation>
+        <translation type="unfinished">(tiada label)</translation>
     </message>
     </context>
 <context>
     <name>TransactionView</name>
-    <message>
-        <source>Comma separated file (*.csv)</source>
-        <translation>Fail dibahagi oleh koma(*.csv)</translation>
-    </message>
-    <message>
-        <source>Label</source>
-        <translation>Label</translation>
-    </message>
     <message>
         <source>Address</source>
-        <translation>Alamat</translation>
+        <translation type="unfinished">Alamat</translation>
     </message>
     <message>
         <source>Exporting Failed</source>
-        <translation>Mengeksport Gagal</translation>
+        <translation type="unfinished">Mengeksport Gagal</translation>
     </message>
     </context>
-<context>
-    <name>UnitDisplayStatusBarControl</name>
-    </context>
-<context>
-    <name>WalletController</name>
-    <message>
-        <source>Close wallet</source>
-        <translation>Tutup Wallet</translation>
-    </message>
-    </context>
-<context>
-    <name>WalletFrame</name>
-    </context>
 <context>
     <name>WalletModel</name>
     <message>
         <source>default wallet</source>
-        <translation>dompet lalai
+        <translation type="unfinished">dompet lalai
 </translation>
     </message>
 </context>
@@ -641,23 +553,23 @@ Alihkan fail data ke dalam tab semasa</translation>
     <name>WalletView</name>
     <message>
         <source>&amp;Export</source>
-        <translation>&amp;Eksport</translation>
+        <translation type="unfinished">&amp;Eksport</translation>
     </message>
     <message>
         <source>Export the data in the current tab to a file</source>
-        <translation>
+        <translation type="unfinished">
 Alihkan fail data ke dalam tab semasa</translation>
     </message>
+    <message>
+        <source>Error</source>
+        <translation type="unfinished">Ralat</translation>
+    </message>
     </context>
 <context>
     <name>bitcoin-core</name>
-    <message>
-        <source>Loading wallet...</source>
-        <translation>Sedang baca wallet...</translation>
-    </message>
     <message>
         <source>Done loading</source>
-        <translation>Baca Selesai</translation>
+        <translation type="unfinished">Baca Selesai</translation>
     </message>
-</context>
+    </context>
 </TS>
\ No newline at end of file
