@@ -1,9 +1,9 @@
-<TS language="ur" version="2.1">
+<TS version="2.1" language="ur">
 <context>
     <name>AddressBookPage</name>
     <message>
         <source>Right-click to edit address or label</source>
-        <translation>پتہ یا لیبل کی تصیح کیلیئے داہنا کلک</translation>
+        <translation type="unfinished">پتہ یا لیبل کی تصیح کیلیئے داہنا کلک</translation>
     </message>
     <message>
         <source>Create a new address</source>
@@ -11,7 +11,7 @@
     </message>
     <message>
         <source>&amp;New</source>
-        <translation>&amp;نیا</translation>
+        <translation type="unfinished">&amp;نیا</translation>
     </message>
     <message>
         <source>Copy the currently selected address to the system clipboard</source>
@@ -19,11 +19,11 @@
     </message>
     <message>
         <source>&amp;Copy</source>
-        <translation>نقل</translation>
+        <translation type="unfinished">نقل</translation>
     </message>
     <message>
         <source>C&amp;lose</source>
-        <translation>بند</translation>
+        <translation type="unfinished">بند</translation>
     </message>
     <message>
         <source>Delete the currently selected address from the list</source>
@@ -31,7 +31,7 @@
     </message>
     <message>
         <source>Enter address or label to search</source>
-        <translation>پتہ یا لیبل تلاشی کے لئے درج کریں</translation>
+        <translation type="unfinished">پتہ یا لیبل تلاشی کے لئے درج کریں</translation>
     </message>
     <message>
         <source>Export the data in the current tab to a file</source>
@@ -47,75 +47,77 @@
     </message>
     <message>
         <source>Choose the address to send coins to</source>
-        <translation>کوئین وصول کرنے والے کا پتہ</translation>
+        <translation type="unfinished">کوئین وصول کرنے والے کا پتہ</translation>
     </message>
     <message>
         <source>Choose the address to receive coins with</source>
-        <translation>کوئین وصول کرنے والے کا پتہ</translation>
+        <translation type="unfinished">کوئین وصول کرنے والے کا پتہ</translation>
     </message>
     <message>
         <source>C&amp;hoose</source>
-        <translation>چننا</translation>
+        <translation type="unfinished">چننا</translation>
     </message>
     <message>
         <source>Sending addresses</source>
-        <translation>پتے ارسال کیے جارہے ہیں</translation>
+        <translation type="unfinished">پتے ارسال کیے جارہے ہیں</translation>
     </message>
     <message>
         <source>Receiving addresses</source>
-        <translation>پتے موصول ہورہے ہیں</translation>
+        <translation type="unfinished">پتے موصول ہورہے ہیں</translation>
     </message>
     <message>
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
-        <translation>یہ آپ کے ادائیگی بھیجنے کے لئے بٹ کوائن ایڈریس ہیں.سکے بھیجنے سے پہلے ہمیشہ رقم اور وصول کنندہ پتہ چیک کریں۔</translation>
+        <translation type="unfinished">یہ آپ کے ادائیگی بھیجنے کے لئے بٹ کوائن ایڈریس ہیں.سکے بھیجنے سے پہلے ہمیشہ رقم اور وصول کنندہ پتہ چیک کریں۔</translation>
     </message>
     <message>
         <source>These are your Bitcoin addresses for receiving payments. Use the 'Create new receiving address' button in the receive tab to create new addresses.
 Signing is only possible with addresses of the type 'legacy'.</source>
-        <translation>ادائیگیوں کے لئے آپ کے بٹ کوائن ایڈریس ہیں۔ نئے پتے بنانے کے لئے وصول کنندہ ٹیب میں 'نیا وصول کنندہ پتہ بنائیں' بٹن کا استعمال کریں۔دستخط صرف 'میراثی' قسم کے پتے کے ساتھ ہی ممکن ہے۔</translation>
+        <translation type="unfinished">ادائیگیوں کے لئے آپ کے بٹ کوائن ایڈریس ہیں۔ نئے پتے بنانے کے لئے وصول کنندہ ٹیب میں 'نیا وصول کنندہ پتہ بنائیں' بٹن کا استعمال کریں۔دستخط صرف 'میراثی' قسم کے پتے کے ساتھ ہی ممکن ہے۔</translation>
     </message>
     <message>
         <source>&amp;Copy Address</source>
-        <translation>&amp;پتا نقل کریں</translation>
+        <translation type="unfinished">&amp;پتا نقل کریں</translation>
     </message>
     <message>
         <source>Copy &amp;Label</source>
-        <translation>&amp;لیبل نقل کریں</translation>
+        <translation type="unfinished">&amp;لیبل نقل کریں</translation>
     </message>
     <message>
         <source>&amp;Edit</source>
-        <translation>&amp;تدوین</translation>
+        <translation type="unfinished">&amp;تدوین</translation>
     </message>
     <message>
         <source>Export Address List</source>
-        <translation>پتا فہرست ایکسپورٹ کریں</translation>
+        <translation type="unfinished">پتا فہرست ایکسپورٹ کریں</translation>
     </message>
     <message>
-        <source>Comma separated file (*.csv)</source>
-        <translation>کاما سے جدا فائلیں (*.csv)</translation>
-    </message>
-    <message>
-        <source>Exporting Failed</source>
-        <translation>ایکسپورٹ ناکام ہوا</translation>
+        <source>Comma separated file</source>
+        <extracomment>Expanded name of the CSV file format. See https://en.wikipedia.org/wiki/Comma-separated_values</extracomment>
+        <translation type="unfinished">کوما سے الگ فائل</translation>
     </message>
     <message>
         <source>There was an error trying to save the address list to %1. Please try again.</source>
-        <translation>پتا فہرست محفوظ کرتے ہوئے %1 نقص کا سامنا ہوا۔ دوبارہ کوشش کریں۔</translation>
+        <extracomment>An error message. %1 is a stand-in argument for the name of the file we attempted to save to.</extracomment>
+        <translation type="unfinished">پتا فہرست محفوظ کرتے ہوئے %1 نقص کا سامنا ہوا۔ دوبارہ کوشش کریں۔</translation>
+    </message>
+    <message>
+        <source>Exporting Failed</source>
+        <translation type="unfinished">ایکسپورٹ ناکام ہوا</translation>
     </message>
 </context>
 <context>
     <name>AddressTableModel</name>
     <message>
         <source>Label</source>
-        <translation>لیبل</translation>
+        <translation type="unfinished">لیبل</translation>
     </message>
     <message>
         <source>Address</source>
-        <translation>پتہ</translation>
+        <translation type="unfinished">پتہ</translation>
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(کوئی لیبل نہیں)</translation>
+        <translation type="unfinished">(کوئی لیبل نہیں)</translation>
     </message>
 </context>
 <context>
@@ -138,130 +140,198 @@ Signing is only possible with addresses of the type 'legacy'.</source>
     </message>
     <message>
         <source>Show passphrase</source>
-        <translation>پاسفریز دکھائیں</translation>
+        <translation type="unfinished">پاسفریز دکھائیں</translation>
     </message>
     <message>
         <source>Encrypt wallet</source>
-        <translation>بٹوے کو خفیہ کریں</translation>
+        <translation type="unfinished">بٹوے کو خفیہ کریں</translation>
     </message>
     <message>
         <source>This operation needs your wallet passphrase to unlock the wallet.</source>
-        <translation>پرس کو غیر مقفل کرنے کے لئے اس آپریشن کو آپ کے بٹوے کا پاسفریز درکار ہے۔</translation>
+        <translation type="unfinished">پرس کو غیر مقفل کرنے کے لئے اس آپریشن کو آپ کے بٹوے کا پاسفریز درکار ہے۔</translation>
     </message>
     <message>
         <source>Unlock wallet</source>
-        <translation>بستہ کھولیں</translation>
-    </message>
-    <message>
-        <source>This operation needs your wallet passphrase to decrypt the wallet.</source>
-        <translation>اس آپریشن کو بٹوے کو  کھولنے کے لئے  یا ڈکرپٹ کرنے کے لئے آپ کے بٹوے کا پاس فراز ضروری ہے۔</translation>
-    </message>
-    <message>
-        <source>Decrypt wallet</source>
-        <translation>ڈکرپٹ والیٹ</translation>
+        <translation type="unfinished">بستہ کھولیں</translation>
     </message>
     <message>
         <source>Change passphrase</source>
-        <translation>پاسفریز تبدیل کریں</translation>
+        <translation type="unfinished">پاسفریز تبدیل کریں</translation>
     </message>
     <message>
         <source>Confirm wallet encryption</source>
-        <translation>پرس کی خفیہ کاری کی تصدیق کریں</translation>
+        <translation type="unfinished">پرس کی خفیہ کاری کی تصدیق کریں</translation>
     </message>
     <message>
         <source>Warning: If you encrypt your wallet and lose your passphrase, you will &lt;b&gt;LOSE ALL OF YOUR BITCOINS&lt;/b&gt;!</source>
-        <translation>انتباہ: اگر آپ اپنا بٹوہ انکرپٹ کرتے ہیں اور اپنا پاس فریز کھو دیتے ہیں تو ، آپ اپنے تمام بٹکوئنز کھو دیں گے.</translation>
+        <translation type="unfinished">انتباہ: اگر آپ اپنا بٹوہ انکرپٹ کرتے ہیں اور اپنا پاس فریز کھو دیتے ہیں تو ، آپ اپنے تمام بٹکوئنز کھو دیں گے.</translation>
     </message>
     <message>
         <source>Are you sure you wish to encrypt your wallet?</source>
-        <translation>کیا آپ واقعی اپنے بٹوے کو خفیہ کرنا چاہتے ہیں؟</translation>
+        <translation type="unfinished">کیا آپ واقعی اپنے بٹوے کو خفیہ کرنا چاہتے ہیں؟</translation>
     </message>
     <message>
         <source>Wallet encrypted</source>
-        <translation>بٹوے کو خفیہ کردہ</translation>
+        <translation type="unfinished">بٹوے کو خفیہ کردہ</translation>
     </message>
     <message>
         <source>Enter the new passphrase for the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
-        <translation>پرس کے لئے نیا پاسفریج درج کریں۔ براہ کرم دس یا زیادہ بے ترتیب حرفوں ، یا آٹھ یا زیادہ الفاظ کا پاس فریز استعمال کریں۔</translation>
+        <translation type="unfinished">پرس کے لئے نیا پاسفریج درج کریں۔ براہ کرم دس یا زیادہ بے ترتیب حرفوں ، یا آٹھ یا زیادہ الفاظ کا پاس فریز استعمال کریں۔</translation>
     </message>
     <message>
         <source>Enter the old passphrase and new passphrase for the wallet.</source>
-        <translation>پرس کے لئے پرانا پاسفریج اور نیا پاسفریز درج کریں۔</translation>
+        <translation type="unfinished">پرس کے لئے پرانا پاسفریج اور نیا پاسفریز درج کریں۔</translation>
     </message>
     <message>
         <source>Remember that encrypting your wallet cannot fully protect your bitcoins from being stolen by malware infecting your computer.</source>
-        <translation>یاد رکھیں کہ آپ کے پرس کو خفیہ کرنا آپ کے بٹ کوائنز کو میلویئر/چور سے آپ کے کمپیوٹر میں انفیکشن لگانے کے ذریعہ چوری ہونے سے پوری طرح محفوظ نہیں رکھ سکتا ہے۔</translation>
+        <translation type="unfinished">یاد رکھیں کہ آپ کے پرس کو خفیہ کرنا آپ کے بٹ کوائنز کو میلویئر/چور سے آپ کے کمپیوٹر میں انفیکشن لگانے کے ذریعہ چوری ہونے سے پوری طرح محفوظ نہیں رکھ سکتا ہے۔</translation>
     </message>
     <message>
         <source>Wallet to be encrypted</source>
-        <translation>بٹوے کو خفیہ کرنا ہے</translation>
+        <translation type="unfinished">بٹوے کو خفیہ کرنا ہے</translation>
     </message>
     <message>
         <source>Your wallet is about to be encrypted. </source>
-        <translation>آپ کا پرس خفیہ ہونے والا ہے۔</translation>
+        <translation type="unfinished">آپ کا پرس خفیہ ہونے والا ہے۔</translation>
     </message>
     <message>
         <source>Your wallet is now encrypted. </source>
-        <translation>آپ کا پرس اب خفیہ ہے۔</translation>
+        <translation type="unfinished">آپ کا پرس اب خفیہ ہے۔</translation>
     </message>
     <message>
         <source>IMPORTANT: Any previous backups you have made of your wallet file should be replaced with the newly generated, encrypted wallet file. For security reasons, previous backups of the unencrypted wallet file will become useless as soon as you start using the new, encrypted wallet.</source>
-        <translation>اہم: کسی بھی پچھلے بیک اپ کو جو آپ نے اپنی بٹوے فائل سے بنایا ہے اس کی جگہ نئی تیار کردہ ، خفیہ کردہ والیٹ فائل کی جگہ لینا چاہئے۔ سیکیورٹی وجوہات کی بناء پر ، بغیر خفیہ کردہ والیٹ فائل کا پچھلا بیک اپ بیکار ہوجائے گا جیسے ہی آپ نیا ، خفیہ کردہ پرس استعمال کرنا شروع کردیں گے۔</translation>
+        <translation type="unfinished">اہم: کسی بھی پچھلے بیک اپ کو جو آپ نے اپنی بٹوے فائل سے بنایا ہے اس کی جگہ نئی تیار کردہ ، خفیہ کردہ والیٹ فائل کی جگہ لینا چاہئے۔ سیکیورٹی وجوہات کی بناء پر ، بغیر خفیہ کردہ والیٹ فائل کا پچھلا بیک اپ بیکار ہوجائے گا جیسے ہی آپ نیا ، خفیہ کردہ پرس استعمال کرنا شروع کردیں گے۔</translation>
     </message>
     <message>
         <source>Wallet encryption failed</source>
-        <translation>والیٹ کی خفیہ کاری ناکام ہوگئی</translation>
+        <translation type="unfinished">والیٹ کی خفیہ کاری ناکام ہوگئی</translation>
     </message>
     <message>
         <source>Wallet encryption failed due to an internal error. Your wallet was not encrypted.</source>
-        <translation>اندرونی خامی کے سبب والیٹ کی خفیہ کاری ناکام ہوگئی۔ آپ کا پرس خفیہ نہیں ہوا تھا۔</translation>
+        <translation type="unfinished">اندرونی خامی کے سبب والیٹ کی خفیہ کاری ناکام ہوگئی۔ آپ کا پرس خفیہ نہیں ہوا تھا۔</translation>
     </message>
     <message>
         <source>The supplied passphrases do not match.</source>
-        <translation>فراہم کردہ پاسفریز مماثل نہیں ہیں۔</translation>
+        <translation type="unfinished">فراہم کردہ پاسفریز مماثل نہیں ہیں۔</translation>
     </message>
     <message>
         <source>Wallet unlock failed</source>
-        <translation>والیٹ انلاک ناکام ہوگیا</translation>
+        <translation type="unfinished">والیٹ انلاک ناکام ہوگیا</translation>
     </message>
     <message>
         <source>The passphrase entered for the wallet decryption was incorrect.</source>
-        <translation>بٹوے کی ڈکرپشن کے لئے درج کردہ پاس فریس غلط تھا۔</translation>
-    </message>
-    <message>
-        <source>Wallet decryption failed</source>
-        <translation>بٹوے کی ڈکرپشن ناکام ہوگئ</translation>
+        <translation type="unfinished">بٹوے کی ڈکرپشن کے لئے درج کردہ پاس فریس غلط تھا۔</translation>
     </message>
     <message>
         <source>Wallet passphrase was successfully changed.</source>
-        <translation>والیٹ کا پاسفریز کامیابی کے ساتھ تبدیل کردیا گیا تھا۔</translation>
+        <translation type="unfinished">والیٹ کا پاسفریز کامیابی کے ساتھ تبدیل کردیا گیا تھا۔</translation>
     </message>
     <message>
         <source>Warning: The Caps Lock key is on!</source>
-        <translation>انتباہ: کیپس لاک کی آن ہے!</translation>
+        <translation type="unfinished">انتباہ: کیپس لاک کی آن ہے!</translation>
     </message>
 </context>
 <context>
     <name>BanTableModel</name>
     <message>
         <source>IP/Netmask</source>
-        <translation>آئی پی / نیٹ ماسک</translation>
+        <translation type="unfinished">آئی پی / نیٹ ماسک</translation>
     </message>
     <message>
         <source>Banned Until</source>
-        <translation>تک پابندی عائد</translation>
+        <translation type="unfinished">تک پابندی عائد</translation>
     </message>
 </context>
+<context>
+    <name>BitcoinApplication</name>
+    <message>
+        <source>Runaway exception</source>
+        <translation type="unfinished">بھگوڑے رعایت</translation>
+    </message>
+    <message>
+        <source>A fatal error occurred. %1 can no longer continue safely and will quit.</source>
+        <translation type="unfinished">ایک مہلک خرابی واقع ہوئی ہے۔  %1 اب مزید سلامتی سے جاری نہیں رہ سکتا ہے اور چھوڑ دے گا۔</translation>
+    </message>
+    <message>
+        <source>Internal error</source>
+        <translation type="unfinished">داخلی خامی</translation>
+    </message>
+    <message>
+        <source>An internal error occurred. %1 will attempt to continue safely. This is an unexpected bug which can be reported as described below.</source>
+        <translation type="unfinished">ایک داخلی خامی پیش آگئی۔ %1 محفوظ طریقے سے جاری رکھنے کی کوشش کریں گے۔ یہ ایک غیر متوقع مسئلہ ہے جس کی اطلاع ذیل میں دی جاسکتی ہے۔</translation>
+    </message>
+</context>
+<context>
+    <name>QObject</name>
+    <message>
+        <source>Error: Specified data directory "%1" does not exist.</source>
+        <translation type="unfinished">خرابی: مخصوص ڈیٹا ڈائریکٹری ""  %1 موجود نہیں ہے۔</translation>
+    </message>
+    <message>
+        <source>Error: Cannot parse configuration file: %1.</source>
+        <translation type="unfinished">خرابی: کنفگریشن فائل کا تجزیہ نہیں کیا جاسکتا۔%1.</translation>
+    </message>
+    <message>
+        <source>Error: %1</source>
+        <translation type="unfinished">خرابی:%1</translation>
+    </message>
+    <message>
+        <source>Error initializing settings: %1</source>
+        <translation type="unfinished">ترتیبات کو شروع کرنے میں خرابی:%1</translation>
+    </message>
+    <message>
+        <source>%1 didn't yet exit safely…</source>
+        <translation type="unfinished">%1ابھی تک سلامتی سے باہر نہیں نکلا…</translation>
+    </message>
+    <message>
+        <source>Amount</source>
+        <translation type="unfinished">رقم</translation>
+    </message>
+    <message numerus="yes">
+        <source>%n second(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n minute(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n hour(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n day(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n week(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%n year(s)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    </context>
 <context>
     <name>BitcoinGUI</name>
-    <message>
-        <source>Sign &amp;message...</source>
-        <translation>سائن اور پیغام ...</translation>
-    </message>
-    <message>
-        <source>Synchronizing with network...</source>
-        <translation>نیٹ ورک کے ساتھ ہم وقت سازی ہو رہی ہے…</translation>
-    </message>
     <message>
         <source>&amp;Overview</source>
         <translation>اور جائزہ</translation>
@@ -288,11 +358,11 @@ Signing is only possible with addresses of the type 'legacy'.</source>
     </message>
     <message>
         <source>&amp;About %1</source>
-        <translation>&amp;معلومات%1</translation>
+        <translation type="unfinished">&amp;معلومات%1</translation>
     </message>
     <message>
         <source>Show information about %1</source>
-        <translation>%1 کے بارے میں معلومات دکھایں</translation>
+        <translation type="unfinished">%1 کے بارے میں معلومات دکھایں</translation>
     </message>
     <message>
         <source>About &amp;Qt</source>
@@ -304,23 +374,24 @@ Signing is only possible with addresses of the type 'legacy'.</source>
     </message>
     <message>
         <source>Modify configuration options for %1</source>
-        <translation>%1 اختیارات کے لئےترتیب ترمیم کریں</translation>
+        <translation type="unfinished">%1 اختیارات کے لئےترتیب ترمیم کریں</translation>
     </message>
     <message>
         <source>Create a new wallet</source>
-        <translation>ایک نیا پرس بنائیں</translation>
+        <translation type="unfinished">ایک نیا پرس بنائیں</translation>
     </message>
     <message>
         <source>Wallet:</source>
-        <translation>پرس:</translation>
+        <translation type="unfinished">پرس:</translation>
     </message>
     <message>
         <source>Network activity disabled.</source>
-        <translation>نیٹ ورک کی سرگرمی غیر فعال ہے۔</translation>
+        <extracomment>A substring of the tooltip.</extracomment>
+        <translation type="unfinished">نیٹ ورک کی سرگرمی غیر فعال ہے۔</translation>
     </message>
     <message>
         <source>Proxy is &lt;b&gt;enabled&lt;/b&gt;: %1</source>
-        <translation>پراکسی فعال ہے:%1</translation>
+        <translation type="unfinished">پراکسی &lt;b&gt;فعال&lt;/b&gt; ہے:%1</translation>
     </message>
     <message>
         <source>Send coins to a Bitcoin address</source>
@@ -342,6 +413,10 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>&amp;Receive</source>
         <translation>&amp;وصول کریں</translation>
     </message>
+    <message>
+        <source>&amp;Options…</source>
+        <translation type="unfinished">&amp;اختیارات…</translation>
+    </message>
     <message>
         <source>&amp;Show / Hide</source>
         <translation>&amp;دکھانا چھپانا</translation>
@@ -351,51 +426,88 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <translation>مین ونڈو دکھائیں یا چھپائیں</translation>
     </message>
     <message>
-        <source>Encrypt the private keys that belong to your wallet</source>
-        <translation>Encrypt the private keys that belong to your wallet</translation>
+        <source>&amp;Encrypt Wallet…</source>
+        <translation type="unfinished">&amp; والیٹ کو خفیہ کریں…</translation>
+    </message>
+    <message>
+        <source>&amp;Backup Wallet…</source>
+        <translation type="unfinished">&amp; بیک اپ والیٹ…</translation>
+    </message>
+    <message>
+        <source>&amp;Change Passphrase…</source>
+        <translation type="unfinished">&amp; بیک اپ والیٹ…</translation>
+    </message>
+    <message>
+        <source>Sign &amp;message…</source>
+        <translation type="unfinished">سائن اور پیغام…</translation>
     </message>
     <message>
         <source>Sign messages with your Bitcoin addresses to prove you own them</source>
         <translation>اپنے ویکیپیڈیا پتوں کے ساتھ پیغامات پر دستخط کریں تاکہ آپ ان کے مالک ہوں</translation>
     </message>
+    <message>
+        <source>&amp;Verify message…</source>
+        <translation type="unfinished">پیغام کی توثیق کریں…</translation>
+    </message>
     <message>
         <source>Verify messages to ensure they were signed with specified Bitcoin addresses</source>
         <translation>پیغامات کی توثیق کریں تاکہ یہ یقینی بن سکے کہ ان پر بٹ کوائن کے مخصوص پتوں پر دستخط ہوئے ہیں</translation>
     </message>
+    <message>
+        <source>Load PSBT from clipboard…</source>
+        <translation type="unfinished">کلپ بورڈ سے پی ایس بی ٹی لوڈ کریں…</translation>
+    </message>
+    <message>
+        <source>Open &amp;URI…</source>
+        <translation type="unfinished">کھولیں اور یو آر آئی…</translation>
+    </message>
+    <message>
+        <source>Close Wallet…</source>
+        <translation type="unfinished">پرس بند کریں…</translation>
+    </message>
+    <message numerus="yes">
+        <source>Processed %n block(s) of transaction history.</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
     <message>
         <source>Error</source>
         <translation>نقص</translation>
     </message>
+    <message numerus="yes">
+        <source>%n active connection(s) to Bitcoin network.</source>
+        <extracomment>A substring of the tooltip.</extracomment>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
     <message>
         <source>Error: %1</source>
-        <translation>خرابی:%1</translation>
+        <translation type="unfinished">خرابی:%1</translation>
     </message>
     </context>
 <context>
     <name>CoinControlDialog</name>
     <message>
         <source>Amount:</source>
-        <translation>رقم:</translation>
+        <translation type="unfinished">رقم:</translation>
     </message>
     <message>
         <source>Amount</source>
-        <translation>رقم</translation>
+        <translation type="unfinished">رقم</translation>
     </message>
     <message>
         <source>Date</source>
-        <translation>تاریخ</translation>
+        <translation type="unfinished">تاریخ</translation>
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(کوئی لیبل نہیں)</translation>
+        <translation type="unfinished">(کوئی لیبل نہیں)</translation>
     </message>
     </context>
-<context>
-    <name>CreateWalletActivity</name>
-    </context>
-<context>
-    <name>CreateWalletDialog</name>
-    </context>
 <context>
     <name>EditAddressDialog</name>
     <message>
@@ -407,232 +519,190 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <translation>پتہ</translation>
     </message>
     </context>
-<context>
-    <name>FreespaceChecker</name>
-    </context>
-<context>
-    <name>HelpMessageDialog</name>
-    </context>
 <context>
     <name>Intro</name>
+    <message numerus="yes">
+        <source>(sufficient to restore backups %n day(s) old)</source>
+        <extracomment>Explanatory text on the capability of the current prune target.</extracomment>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
     <message>
         <source>Error</source>
         <translation>نقص</translation>
     </message>
     </context>
-<context>
-    <name>ModalOverlay</name>
-    </context>
-<context>
-    <name>OpenURIDialog</name>
-    </context>
-<context>
-    <name>OpenWalletActivity</name>
-    </context>
 <context>
     <name>OptionsDialog</name>
     <message>
         <source>Error</source>
-        <translation>نقص</translation>
+        <translation type="unfinished">نقص</translation>
     </message>
     </context>
-<context>
-    <name>OverviewPage</name>
-    </context>
-<context>
-    <name>PSBTOperationsDialog</name>
-    </context>
-<context>
-    <name>PaymentServer</name>
-    </context>
 <context>
     <name>PeerTableModel</name>
-    </context>
-<context>
-    <name>QObject</name>
     <message>
-        <source>Amount</source>
-        <translation>رقم</translation>
+        <source>Address</source>
+        <extracomment>Title of Peers Table column which contains the IP/Onion/I2P address of the connected peer.</extracomment>
+        <translation type="unfinished">پتہ</translation>
     </message>
-    <message>
-        <source>Error: Specified data directory "%1" does not exist.</source>
-        <translation>خرابی: مخصوص ڈیٹا ڈائریکٹری ""  %1 موجود نہیں ہے۔</translation>
-    </message>
-    <message>
-        <source>Error: Cannot parse configuration file: %1.</source>
-        <translation>خرابی: کنفگریشن فائل کا تجزیہ نہیں کیا جاسکتا۔%1.</translation>
-    </message>
-    <message>
-        <source>Error: %1</source>
-        <translation>خرابی:%1</translation>
-    </message>
-    <message>
-        <source>Error initializing settings: %1</source>
-        <translation>ترتیبات کو شروع کرنے میں خرابی:%1</translation>
-    </message>
-    </context>
-<context>
-    <name>QRImageWidget</name>
-    </context>
-<context>
-    <name>RPCConsole</name>
-    </context>
-<context>
-    <name>ReceiveCoinsDialog</name>
     </context>
 <context>
     <name>ReceiveRequestDialog</name>
     <message>
         <source>Amount:</source>
-        <translation>رقم:</translation>
+        <translation type="unfinished">رقم:</translation>
     </message>
     <message>
         <source>Wallet:</source>
-        <translation>پرس:</translation>
+        <translation type="unfinished">پرس:</translation>
     </message>
     <message>
         <source>Copy &amp;Address</source>
-        <translation>کاپی پتہ</translation>
+        <translation type="unfinished">کاپی پتہ</translation>
     </message>
     </context>
 <context>
     <name>RecentRequestsTableModel</name>
     <message>
         <source>Date</source>
-        <translation>تاریخ</translation>
+        <translation type="unfinished">تاریخ</translation>
     </message>
     <message>
         <source>Label</source>
-        <translation>لیبل</translation>
+        <translation type="unfinished">لیبل</translation>
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(کوئی لیبل نہیں)</translation>
+        <translation type="unfinished">(کوئی لیبل نہیں)</translation>
     </message>
     </context>
 <context>
     <name>SendCoinsDialog</name>
     <message>
         <source>Insufficient funds!</source>
-        <translation>ناکافی فنڈز</translation>
+        <translation type="unfinished">ناکافی فنڈز</translation>
     </message>
     <message>
         <source>Amount:</source>
-        <translation>رقم:</translation>
+        <translation type="unfinished">رقم:</translation>
     </message>
     <message>
         <source>Balance:</source>
         <translation>بیلنس:</translation>
     </message>
+    <message numerus="yes">
+        <source>Estimated to begin confirmation within %n block(s).</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
     <message>
         <source>(no label)</source>
-        <translation>(کوئی لیبل نہیں)</translation>
+        <translation type="unfinished">(کوئی لیبل نہیں)</translation>
     </message>
 </context>
-<context>
-    <name>SendCoinsEntry</name>
-    </context>
-<context>
-    <name>ShutdownWindow</name>
-    </context>
-<context>
-    <name>SignVerifyMessageDialog</name>
-    </context>
-<context>
-    <name>TrafficGraphWidget</name>
-    </context>
 <context>
     <name>TransactionDesc</name>
+    <message numerus="yes">
+        <source>Open for %n more block(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
     <message>
         <source>Date</source>
-        <translation>تاریخ</translation>
+        <translation type="unfinished">تاریخ</translation>
+    </message>
+    <message numerus="yes">
+        <source>matures in %n more block(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
     <message>
         <source>Amount</source>
-        <translation>رقم</translation>
+        <translation type="unfinished">رقم</translation>
     </message>
     </context>
-<context>
-    <name>TransactionDescDialog</name>
-    </context>
 <context>
     <name>TransactionTableModel</name>
     <message>
         <source>Date</source>
-        <translation>تاریخ</translation>
+        <translation type="unfinished">تاریخ</translation>
     </message>
     <message>
         <source>Label</source>
-        <translation>لیبل</translation>
+        <translation type="unfinished">لیبل</translation>
+    </message>
+    <message numerus="yes">
+        <source>Open for %n more block(s)</source>
+        <translation>
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(کوئی لیبل نہیں)</translation>
+        <translation type="unfinished">(کوئی لیبل نہیں)</translation>
     </message>
     </context>
 <context>
     <name>TransactionView</name>
     <message>
-        <source>Other</source>
-        <translation>Other</translation>
-    </message>
-    <message>
-        <source>Comma separated file (*.csv)</source>
-        <translation>کاما سے جدا فائلیں (*.csv)</translation>
+        <source>Comma separated file</source>
+        <extracomment>Expanded name of the CSV file format. See https://en.wikipedia.org/wiki/Comma-separated_values</extracomment>
+        <translation type="unfinished">کوما سے الگ فائل</translation>
     </message>
     <message>
         <source>Date</source>
-        <translation>تاریخ</translation>
+        <translation type="unfinished">تاریخ</translation>
     </message>
     <message>
         <source>Label</source>
-        <translation>لیبل</translation>
+        <translation type="unfinished">لیبل</translation>
     </message>
     <message>
         <source>Address</source>
-        <translation>پتہ</translation>
+        <translation type="unfinished">پتہ</translation>
     </message>
     <message>
         <source>Exporting Failed</source>
-        <translation>ایکسپورٹ ناکام ہوا</translation>
+        <translation type="unfinished">ایکسپورٹ ناکام ہوا</translation>
     </message>
     </context>
-<context>
-    <name>UnitDisplayStatusBarControl</name>
-    </context>
-<context>
-    <name>WalletController</name>
-    </context>
 <context>
     <name>WalletFrame</name>
     <message>
         <source>Create a new wallet</source>
-        <translation>ایک نیا پرس بنائیں</translation>
+        <translation type="unfinished">ایک نیا پرس بنائیں</translation>
     </message>
 </context>
-<context>
-    <name>WalletModel</name>
-    </context>
 <context>
     <name>WalletView</name>
     <message>
         <source>&amp;Export</source>
-        <translation>برآمد</translation>
+        <translation type="unfinished">برآمد</translation>
     </message>
     <message>
         <source>Export the data in the current tab to a file</source>
-        <translation>موجودہ ڈیٹا کو فائیل میں محفوظ کریں</translation>
+        <translation type="unfinished">موجودہ ڈیٹا کو فائیل میں محفوظ کریں</translation>
     </message>
     <message>
         <source>Error</source>
-        <translation>نقص</translation>
+        <translation type="unfinished">نقص</translation>
     </message>
     </context>
 <context>
     <name>bitcoin-core</name>
     <message>
         <source>Insufficient funds</source>
-        <translation>ناکافی فنڈز</translation>
+        <translation type="unfinished">ناکافی فنڈز</translation>
     </message>
     </context>
 </TS>
\ No newline at end of file
