@@ -1,11 +1,10 @@
-// Copyright (c) 2012-2018 The Bitcoin Core developers
+// Copyright (c) 2012-2020 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
 #include <dbwrapper.h>
+#include <test/util/setup_common.h>
 #include <uint256.h>
-#include <random.h>
-#include <test/test_bitcoin.h>
 
 #include <memory>
 
@@ -26,10 +25,10 @@ BOOST_FIXTURE_TEST_SUITE(dbwrapper_tests, BasicTestingSetup)
 BOOST_AUTO_TEST_CASE(dbwrapper)
 {
     // Perform tests both obfuscated and non-obfuscated.
-    for (bool obfuscate : {false, true}) {
-        fs::path ph = SetDataDir(std::string("dbwrapper").append(obfuscate ? "_true" : "_false"));
+    for (const bool obfuscate : {false, true}) {
+        fs::path ph = m_args.GetDataDirBase() / (obfuscate ? "dbwrapper_obfuscate_true" : "dbwrapper_obfuscate_false");
         CDBWrapper dbw(ph, (1 << 20), true, false, obfuscate);
-        char key = 'k';
+        uint8_t key{'k'};
         uint256 in = InsecureRand256();
         uint256 res;
 
@@ -42,19 +41,99 @@ BOOST_AUTO_TEST_CASE(dbwrapper)
     }
 }
 
+BOOST_AUTO_TEST_CASE(dbwrapper_basic_data)
+{
+    // Perform tests both obfuscated and non-obfuscated.
+    for (bool obfuscate : {false, true}) {
+        fs::path ph = m_args.GetDataDirBase() / (obfuscate ? "dbwrapper_1_obfuscate_true" : "dbwrapper_1_obfuscate_false");
+        CDBWrapper dbw(ph, (1 << 20), false, true, obfuscate);
+
+        uint256 res;
+        uint32_t res_uint_32;
+        bool res_bool;
+
+        // Ensure that we're doing real obfuscation when obfuscate=true
+        BOOST_CHECK(obfuscate != is_null_key(dbwrapper_private::GetObfuscateKey(dbw)));
+
+        //Simulate block raw data - "b + block hash"
+        std::string key_block = "b" + InsecureRand256().ToString();
+
+        uint256 in_block = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_block, in_block));
+        BOOST_CHECK(dbw.Read(key_block, res));
+        BOOST_CHECK_EQUAL(res.ToString(), in_block.ToString());
+
+        //Simulate file raw data - "f + file_number"
+        std::string key_file = strprintf("f%04x", InsecureRand32());
+
+        uint256 in_file_info = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_file, in_file_info));
+        BOOST_CHECK(dbw.Read(key_file, res));
+        BOOST_CHECK_EQUAL(res.ToString(), in_file_info.ToString());
+
+        //Simulate transaction raw data - "t + transaction hash"
+        std::string key_transaction = "t" + InsecureRand256().ToString();
+
+        uint256 in_transaction = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_transaction, in_transaction));
+        BOOST_CHECK(dbw.Read(key_transaction, res));
+        BOOST_CHECK_EQUAL(res.ToString(), in_transaction.ToString());
+
+        //Simulate UTXO raw data - "c + transaction hash"
+        std::string key_utxo = "c" + InsecureRand256().ToString();
+
+        uint256 in_utxo = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_utxo, in_utxo));
+        BOOST_CHECK(dbw.Read(key_utxo, res));
+        BOOST_CHECK_EQUAL(res.ToString(), in_utxo.ToString());
+
+        //Simulate last block file number - "l"
+        uint8_t key_last_blockfile_number{'l'};
+        uint32_t lastblockfilenumber = InsecureRand32();
+        BOOST_CHECK(dbw.Write(key_last_blockfile_number, lastblockfilenumber));
+        BOOST_CHECK(dbw.Read(key_last_blockfile_number, res_uint_32));
+        BOOST_CHECK_EQUAL(lastblockfilenumber, res_uint_32);
+
+        //Simulate Is Reindexing - "R"
+        uint8_t key_IsReindexing{'R'};
+        bool isInReindexing = InsecureRandBool();
+        BOOST_CHECK(dbw.Write(key_IsReindexing, isInReindexing));
+        BOOST_CHECK(dbw.Read(key_IsReindexing, res_bool));
+        BOOST_CHECK_EQUAL(isInReindexing, res_bool);
+
+        //Simulate last block hash up to which UXTO covers - 'B'
+        uint8_t key_lastblockhash_uxto{'B'};
+        uint256 lastblock_hash = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_lastblockhash_uxto, lastblock_hash));
+        BOOST_CHECK(dbw.Read(key_lastblockhash_uxto, res));
+        BOOST_CHECK_EQUAL(lastblock_hash, res);
+
+        //Simulate file raw data - "F + filename_number + filename"
+        std::string file_option_tag = "F";
+        uint8_t filename_length = InsecureRandBits(8);
+        std::string filename = "randomfilename";
+        std::string key_file_option = strprintf("%s%01x%s", file_option_tag,filename_length,filename);
+
+        bool in_file_bool = InsecureRandBool();
+        BOOST_CHECK(dbw.Write(key_file_option, in_file_bool));
+        BOOST_CHECK(dbw.Read(key_file_option, res_bool));
+        BOOST_CHECK_EQUAL(res_bool, in_file_bool);
+   }
+}
+
 // Test batch operations
 BOOST_AUTO_TEST_CASE(dbwrapper_batch)
 {
     // Perform tests both obfuscated and non-obfuscated.
-    for (bool obfuscate : {false, true}) {
-        fs::path ph = SetDataDir(std::string("dbwrapper_batch").append(obfuscate ? "_true" : "_false"));
+    for (const bool obfuscate : {false, true}) {
+        fs::path ph = m_args.GetDataDirBase() / (obfuscate ? "dbwrapper_batch_obfuscate_true" : "dbwrapper_batch_obfuscate_false");
         CDBWrapper dbw(ph, (1 << 20), true, false, obfuscate);
 
-        char key = 'i';
+        uint8_t key{'i'};
         uint256 in = InsecureRand256();
-        char key2 = 'j';
+        uint8_t key2{'j'};
         uint256 in2 = InsecureRand256();
-        char key3 = 'k';
+        uint8_t key3{'k'};
         uint256 in3 = InsecureRand256();
 
         uint256 res;
@@ -67,7 +146,7 @@ BOOST_AUTO_TEST_CASE(dbwrapper_batch)
         // Remove key3 before it's even been written
         batch.Erase(key3);
 
-        dbw.WriteBatch(batch);
+        BOOST_CHECK(dbw.WriteBatch(batch));
 
         BOOST_CHECK(dbw.Read(key, res));
         BOOST_CHECK_EQUAL(res.ToString(), in.ToString());
@@ -82,15 +161,15 @@ BOOST_AUTO_TEST_CASE(dbwrapper_batch)
 BOOST_AUTO_TEST_CASE(dbwrapper_iterator)
 {
     // Perform tests both obfuscated and non-obfuscated.
-    for (bool obfuscate : {false, true}) {
-        fs::path ph = SetDataDir(std::string("dbwrapper_iterator").append(obfuscate ? "_true" : "_false"));
+    for (const bool obfuscate : {false, true}) {
+        fs::path ph = m_args.GetDataDirBase() / (obfuscate ? "dbwrapper_iterator_obfuscate_true" : "dbwrapper_iterator_obfuscate_false");
         CDBWrapper dbw(ph, (1 << 20), true, false, obfuscate);
 
         // The two keys are intentionally chosen for ordering
-        char key = 'j';
+        uint8_t key{'j'};
         uint256 in = InsecureRand256();
         BOOST_CHECK(dbw.Write(key, in));
-        char key2 = 'k';
+        uint8_t key2{'k'};
         uint256 in2 = InsecureRand256();
         BOOST_CHECK(dbw.Write(key2, in2));
 
@@ -99,18 +178,18 @@ BOOST_AUTO_TEST_CASE(dbwrapper_iterator)
         // Be sure to seek past the obfuscation key (if it exists)
         it->Seek(key);
 
-        char key_res;
+        uint8_t key_res;
         uint256 val_res;
 
-        it->GetKey(key_res);
-        it->GetValue(val_res);
+        BOOST_REQUIRE(it->GetKey(key_res));
+        BOOST_REQUIRE(it->GetValue(val_res));
         BOOST_CHECK_EQUAL(key_res, key);
         BOOST_CHECK_EQUAL(val_res.ToString(), in.ToString());
 
         it->Next();
 
-        it->GetKey(key_res);
-        it->GetValue(val_res);
+        BOOST_REQUIRE(it->GetKey(key_res));
+        BOOST_REQUIRE(it->GetValue(val_res));
         BOOST_CHECK_EQUAL(key_res, key2);
         BOOST_CHECK_EQUAL(val_res.ToString(), in2.ToString());
 
@@ -123,12 +202,12 @@ BOOST_AUTO_TEST_CASE(dbwrapper_iterator)
 BOOST_AUTO_TEST_CASE(existing_data_no_obfuscate)
 {
     // We're going to share this fs::path between two wrappers
-    fs::path ph = SetDataDir("existing_data_no_obfuscate");
+    fs::path ph = m_args.GetDataDirBase() / "existing_data_no_obfuscate";
     create_directories(ph);
 
     // Set up a non-obfuscated wrapper to write some initial data.
-    std::unique_ptr<CDBWrapper> dbw = MakeUnique<CDBWrapper>(ph, (1 << 10), false, false, false);
-    char key = 'k';
+    std::unique_ptr<CDBWrapper> dbw = std::make_unique<CDBWrapper>(ph, (1 << 10), false, false, false);
+    uint8_t key{'k'};
     uint256 in = InsecureRand256();
     uint256 res;
 
@@ -164,12 +243,12 @@ BOOST_AUTO_TEST_CASE(existing_data_no_obfuscate)
 BOOST_AUTO_TEST_CASE(existing_data_reindex)
 {
     // We're going to share this fs::path between two wrappers
-    fs::path ph = SetDataDir("existing_data_reindex");
+    fs::path ph = m_args.GetDataDirBase() / "existing_data_reindex";
     create_directories(ph);
 
     // Set up a non-obfuscated wrapper to write some initial data.
-    std::unique_ptr<CDBWrapper> dbw = MakeUnique<CDBWrapper>(ph, (1 << 10), false, false, false);
-    char key = 'k';
+    std::unique_ptr<CDBWrapper> dbw = std::make_unique<CDBWrapper>(ph, (1 << 10), false, false, false);
+    uint8_t key{'k'};
     uint256 in = InsecureRand256();
     uint256 res;
 
@@ -199,7 +278,7 @@ BOOST_AUTO_TEST_CASE(existing_data_reindex)
 
 BOOST_AUTO_TEST_CASE(iterator_ordering)
 {
-    fs::path ph = SetDataDir("iterator_ordering");
+    fs::path ph = m_args.GetDataDirBase() / "iterator_ordering";
     CDBWrapper dbw(ph, (1 << 20), true, false, false);
     for (int x=0x00; x<256; ++x) {
         uint8_t key = x;
@@ -216,7 +295,7 @@ BOOST_AUTO_TEST_CASE(iterator_ordering)
         if (x & 1) BOOST_CHECK(dbw.Write(key, value));
     }
 
-    for (int seek_start : {0x00, 0x80}) {
+    for (const int seek_start : {0x00, 0x80}) {
         it->Seek((uint8_t)seek_start);
         for (unsigned int x=seek_start; x<255; ++x) {
             uint8_t key;
@@ -251,24 +330,26 @@ struct StringContentsSerializer {
     }
     StringContentsSerializer& operator+=(const StringContentsSerializer& s) { return *this += s.str; }
 
-    ADD_SERIALIZE_METHODS;
+    template<typename Stream>
+    void Serialize(Stream& s) const
+    {
+        for (size_t i = 0; i < str.size(); i++) {
+            s << uint8_t(str[i]);
+        }
+    }
 
-    template <typename Stream, typename Operation>
-    inline void SerializationOp(Stream& s, Operation ser_action) {
-        if (ser_action.ForRead()) {
-            str.clear();
-            char c = 0;
-            while (true) {
-                try {
-                    READWRITE(c);
-                    str.push_back(c);
-                } catch (const std::ios_base::failure& e) {
-                    break;
-                }
+    template<typename Stream>
+    void Unserialize(Stream& s)
+    {
+        str.clear();
+        uint8_t c{0};
+        while (true) {
+            try {
+                s >> c;
+                str.push_back(c);
+            } catch (const std::ios_base::failure&) {
+                break;
             }
-        } else {
-            for (size_t i = 0; i < str.size(); i++)
-                READWRITE(str[i]);
         }
     }
 };
@@ -277,7 +358,7 @@ BOOST_AUTO_TEST_CASE(iterator_string_ordering)
 {
     char buf[10];
 
-    fs::path ph = SetDataDir("iterator_string_ordering");
+    fs::path ph = m_args.GetDataDirBase() / "iterator_string_ordering";
     CDBWrapper dbw(ph, (1 << 20), true, false, false);
     for (int x=0x00; x<10; ++x) {
         for (int y = 0; y < 10; y++) {
@@ -291,7 +372,7 @@ BOOST_AUTO_TEST_CASE(iterator_string_ordering)
     }
 
     std::unique_ptr<CDBIterator> it(const_cast<CDBWrapper&>(dbw).NewIterator());
-    for (int seek_start : {0, 5}) {
+    for (const int seek_start : {0, 5}) {
         snprintf(buf, sizeof(buf), "%d", seek_start);
         StringContentsSerializer seek_key(buf);
         it->Seek(seek_key);
@@ -317,6 +398,18 @@ BOOST_AUTO_TEST_CASE(iterator_string_ordering)
     }
 }
 
+BOOST_AUTO_TEST_CASE(unicodepath)
+{
+    // Attempt to create a database with a UTF8 character in the path.
+    // On Windows this test will fail if the directory is created using
+    // the ANSI CreateDirectoryA call and the code page isn't UTF8.
+    // It will succeed if created with CreateDirectoryW.
+    fs::path ph = m_args.GetDataDirBase() / "test_runner_₿_🏃_20191128_104644";
+    CDBWrapper dbw(ph, (1 << 20));
+
+    fs::path lockPath = ph / "LOCK";
+    BOOST_CHECK(fs::exists(lockPath));
+}
 
 
 BOOST_AUTO_TEST_SUITE_END()
