@@ -1,33 +1,34 @@
-// Copyright (c) 2019 The Bitcoin Core developers
+// Copyright (c) 2019-2020 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
+#include <test/fuzz/FuzzedDataProvider.h>
 #include <test/fuzz/fuzz.h>
 
 #include <node/psbt.h>
-#include <optional.h>
 #include <psbt.h>
 #include <pubkey.h>
 #include <script/script.h>
 #include <streams.h>
-#include <util/memory.h>
+#include <util/check.h>
 #include <version.h>
 
 #include <cstdint>
+#include <optional>
 #include <string>
 #include <vector>
 
-void initialize()
+void initialize_psbt()
 {
     static const ECCVerifyHandle verify_handle;
 }
 
-void test_one_input(const std::vector<uint8_t>& buffer)
+FUZZ_TARGET_INIT(psbt, initialize_psbt)
 {
+    FuzzedDataProvider fuzzed_data_provider{buffer.data(), buffer.size()};
     PartiallySignedTransaction psbt_mut;
-    const std::string raw_psbt{buffer.begin(), buffer.end()};
     std::string error;
-    if (!DecodeRawPSBT(psbt_mut, raw_psbt, error)) {
+    if (!DecodeRawPSBT(psbt_mut, fuzzed_data_provider.ConsumeRandomLengthString(), error)) {
         return;
     }
     const PartiallySignedTransaction psbt = psbt_mut;
@@ -40,7 +41,7 @@ void test_one_input(const std::vector<uint8_t>& buffer)
 
     (void)psbt.IsNull();
 
-    Optional<CMutableTransaction> tx = psbt.tx;
+    std::optional<CMutableTransaction> tx = psbt.tx;
     if (tx) {
         const CMutableTransaction& mtx = *tx;
         const PartiallySignedTransaction psbt_from_tx{mtx};
@@ -50,6 +51,7 @@ void test_one_input(const std::vector<uint8_t>& buffer)
         (void)PSBTInputSigned(input);
         (void)input.IsNull();
     }
+    (void)CountPSBTUnsignedInputs(psbt);
 
     for (const PSBTOutput& output : psbt.outputs) {
         (void)output.IsNull();
@@ -72,6 +74,20 @@ void test_one_input(const std::vector<uint8_t>& buffer)
         const PartiallySignedTransaction psbt_from_tx{result};
     }
 
+    PartiallySignedTransaction psbt_merge;
+    if (!DecodeRawPSBT(psbt_merge, fuzzed_data_provider.ConsumeRandomLengthString(), error)) {
+        psbt_merge = psbt;
+    }
     psbt_mut = psbt;
-    (void)psbt_mut.Merge(psbt);
+    (void)psbt_mut.Merge(psbt_merge);
+    psbt_mut = psbt;
+    (void)CombinePSBTs(psbt_mut, {psbt_mut, psbt_merge});
+    psbt_mut = psbt;
+    for (unsigned int i = 0; i < psbt_merge.tx->vin.size(); ++i) {
+        (void)psbt_mut.AddInput(psbt_merge.tx->vin[i], psbt_merge.inputs[i]);
+    }
+    for (unsigned int i = 0; i < psbt_merge.tx->vout.size(); ++i) {
+        Assert(psbt_mut.AddOutput(psbt_merge.tx->vout[i], psbt_merge.outputs[i]));
+    }
+    psbt_mut.unknown.insert(psbt_merge.unknown.begin(), psbt_merge.unknown.end());
 }
