@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2019 The Bitcoin Core developers
+// Copyright (c) 2018-2020 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -15,21 +15,21 @@ BOOST_FIXTURE_TEST_SUITE(init_tests, InitWalletDirTestingSetup)
 BOOST_AUTO_TEST_CASE(walletinit_verify_walletdir_default)
 {
     SetWalletDir(m_walletdir_path_cases["default"]);
-    bool result = m_chain_client->verify();
+    bool result = m_wallet_client->verify();
     BOOST_CHECK(result == true);
     fs::path walletdir = gArgs.GetArg("-walletdir", "");
     fs::path expected_path = fs::canonical(m_walletdir_path_cases["default"]);
-    BOOST_CHECK(walletdir == expected_path);
+    BOOST_CHECK_EQUAL(walletdir, expected_path);
 }
 
 BOOST_AUTO_TEST_CASE(walletinit_verify_walletdir_custom)
 {
     SetWalletDir(m_walletdir_path_cases["custom"]);
-    bool result = m_chain_client->verify();
+    bool result = m_wallet_client->verify();
     BOOST_CHECK(result == true);
     fs::path walletdir = gArgs.GetArg("-walletdir", "");
     fs::path expected_path = fs::canonical(m_walletdir_path_cases["custom"]);
-    BOOST_CHECK(walletdir == expected_path);
+    BOOST_CHECK_EQUAL(walletdir, expected_path);
 }
 
 BOOST_AUTO_TEST_CASE(walletinit_verify_walletdir_does_not_exist)
@@ -37,7 +37,7 @@ BOOST_AUTO_TEST_CASE(walletinit_verify_walletdir_does_not_exist)
     SetWalletDir(m_walletdir_path_cases["nonexistent"]);
     {
         ASSERT_DEBUG_LOG("does not exist");
-        bool result = m_chain_client->verify();
+        bool result = m_wallet_client->verify();
         BOOST_CHECK(result == false);
     }
 }
@@ -47,7 +47,7 @@ BOOST_AUTO_TEST_CASE(walletinit_verify_walletdir_is_not_directory)
     SetWalletDir(m_walletdir_path_cases["file"]);
     {
         ASSERT_DEBUG_LOG("is not a directory");
-        bool result = m_chain_client->verify();
+        bool result = m_wallet_client->verify();
         BOOST_CHECK(result == false);
     }
 }
@@ -57,7 +57,7 @@ BOOST_AUTO_TEST_CASE(walletinit_verify_walletdir_is_not_relative)
     SetWalletDir(m_walletdir_path_cases["relative"]);
     {
         ASSERT_DEBUG_LOG("is a relative path");
-        bool result = m_chain_client->verify();
+        bool result = m_wallet_client->verify();
         BOOST_CHECK(result == false);
     }
 }
@@ -65,21 +65,21 @@ BOOST_AUTO_TEST_CASE(walletinit_verify_walletdir_is_not_relative)
 BOOST_AUTO_TEST_CASE(walletinit_verify_walletdir_no_trailing)
 {
     SetWalletDir(m_walletdir_path_cases["trailing"]);
-    bool result = m_chain_client->verify();
+    bool result = m_wallet_client->verify();
     BOOST_CHECK(result == true);
     fs::path walletdir = gArgs.GetArg("-walletdir", "");
     fs::path expected_path = fs::canonical(m_walletdir_path_cases["default"]);
-    BOOST_CHECK(walletdir == expected_path);
+    BOOST_CHECK_EQUAL(walletdir, expected_path);
 }
 
 BOOST_AUTO_TEST_CASE(walletinit_verify_walletdir_no_trailing2)
 {
     SetWalletDir(m_walletdir_path_cases["trailing2"]);
-    bool result = m_chain_client->verify();
+    bool result = m_wallet_client->verify();
     BOOST_CHECK(result == true);
     fs::path walletdir = gArgs.GetArg("-walletdir", "");
     fs::path expected_path = fs::canonical(m_walletdir_path_cases["default"]);
-    BOOST_CHECK(walletdir == expected_path);
+    BOOST_CHECK_EQUAL(walletdir, expected_path);
 }
 
 BOOST_AUTO_TEST_SUITE_END()
