@@ -1,10 +1,12 @@
-// Copyright (c) 2017-2019 The Bitcoin Core developers
+// Copyright (c) 2017-2020 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
 #ifndef BITCOIN_WALLETINITINTERFACE_H
 #define BITCOIN_WALLETINITINTERFACE_H
 
+class ArgsManager;
+
 struct NodeContext;
 
 class WalletInitInterface {
@@ -12,7 +14,7 @@ public:
     /** Is the wallet component enabled */
     virtual bool HasWalletSupport() const = 0;
     /** Get wallet help string */
-    virtual void AddWalletOptions() const = 0;
+    virtual void AddWalletOptions(ArgsManager& argsman) const = 0;
     /** Check wallet parameter interaction */
     virtual bool ParameterInteraction() const = 0;
     /** Add wallets that should be opened to list of chain clients. */
